/* _sp2m -- Convert Natsuhara's SPIN2 files to those for "pdemag" and "pdirec".
	V2.1 -- minor amendments   0619
	V2.0 -- for old style file (_*.txt) of Natsuhara's spinner 2000
		at Marine Core Center of Kochi University   0315,0812
	V1.2 -- for Natsuhara's old spinner   0903
	Usage:
	  _sp2m [-NH] file1 file2 ...
	Options:
	  N -- for NRM measurements and output format is for "pdirec"
	  H -- shows a help message
	Input files:
	  Natsuhara's old formatted file (_*.txt) of SPIN2, which are
	  either of step-wise demagnetization data from a single sample,
	  or NRM measurements of multiple samples from a single site.
	  Reads <=50 files but use of a wild card "*" is NOT supported.
	Output files:
	  Results are written to stdout with either of the following formats.
	    Default (for use by "pdemag"):
	      $ sample azim plun strik dip (take those only once from 1st line)
	      step M Ic Dc                 (inc & dec are in core frame)
	        ...
	        ...
	    N option (for use by "pdirec"):
	      $ file
	      sample Ic Dc Is Ds It Dt -1.0 A 1 M ("-1.0 A 1" are dummies)
	        ...
	        ...
	Basic idea:
	    The program supposes that the input data are either of (1) series of
	step-wise demagnetization from a single sample (default) or (2) NRM
	measurements of multiple samples from a single site (-N option).
	In the default case, orientation (azim,plun) and bedding attitude
	(strik,dip) are taken ONLY ONCE from the 1st line, and ONLY the data of
	core coordinates (i.e., sample frame) are taken from each step. In the
	case of -N option, orientation and bedding attitude are taken from each
	measurement, and the in situ and bedding corrected directions are
	calculated from the core data (those in the input files are not used).
*/

#include "_sp2m.h"

char Files[NFILES][LFILES];
unsigned Opt = 0;

int main(argc, argv)
int argc;
char *argv[];
{
	int nf, i;

	if ((nf = _sp2m_cmd(argc, argv)) < 1) error(USAGE);
	if ( !(Opt & NRM_BIT) )
		for (i = 0; i < nf; i++) _sp2m_cal(i);
	else
		for (i = 0; i < nf; i++) _sp2m_caln(i);
}
