/* ig_year -- read decimal year or yr-mo-dy, height or radius, from stdin */

#include "igrf.h"

int ig_year(dyear, htgd)
double *dyear, *htgd;
{
	extern unsigned int Opt;
	extern int Gtype, Yr, Mo, Dy;
	char *scont[]={"Altitude in km? (-10 ~ 500 km)\n(Rtn - 0 km, Q - quit)",
		"Geocentric distance in km? (3485~10000 km)\n(Rtn - 6371.2 km, Q - quit)"};
	int c, idum;

	if (!(Opt & YMD_BIT)){
		while ((c=inpd("Decimal year? (Q - quit)", 1, dyear)) != ZERO){
			if (c == -1) continue;
			if (ig_check_dyear(*dyear) == -1) continue;
			break;
		}
		if (c == ZERO) return(-1);
	}
	else {
		while ((c=inpi("Year,Month,Day? (Q - quit)", 3, &Yr, &Mo, &Dy)) != ZERO){
			if (c == -1) continue;
			if (ig_check_yr(Yr) == -1) continue;
			if (ig_check_mody(Mo,Dy) == -1) continue;
			*dyear = ymdtoy(Yr, Mo, Dy, &idum);
			break;
		}
		if (c == ZERO) return(-1);
	}
	if (Gtype == GEODETIC){
		while ((c=inpd(scont[Gtype],1,htgd)) != ZERO){
			if (c == -1){
				*htgd = 0.;
				return(1);
			}
			if (*htgd<-10. || *htgd>500.) continue;
			break;
		}
		if (c == ZERO) return(-1);
	}
	else if (Gtype == GEOCENTRIC) {
		while ((c=inpd(scont[Gtype],1,htgd)) != ZERO){
			if (c == -1){
				*htgd = 6371.2;
				return(1);
			}
			if (*htgd<3485. || *htgd>10000.) continue;
			break;
		}
		if (c == ZERO) return(-1);
	}
	else
		error("Error of Gtype!");
	return(1);
}

/* ig_check_dyear */

int ig_check_dyear(dyr)
double dyr;
{
	if (dyr<1900. || dyr>2026.){
		fprintf(stderr, "** Date span: 1900.0 <= year < 2026.0! **\n");
		return(-1);
	}
	return(1);
}

/* ig_check_yr */

int ig_check_yr(yr)
int yr;
{
	if (yr<1900 || yr>2025){
		fprintf(stderr, "** Year span: 1900 <= year <= 2025! **\n");
		return(-1);
	}
	return(1);
}

/* ig_check_mody */

int ig_check_mody(mo, dy)
int mo, dy;
{
	if (mo<1 || mo>12 || dy<1 || dy>31){
		fprintf(stderr, "** Month or Day must be in error! **\n");
		return(-1);
	}
	return(1);
}
