/* igrf -- Calculation of main field from IGRF-13 up to n=m=10   040720 */

#include <ctype.h>
#include <stdlib.h>
#include <math.h>
#include "igrf.h"

#define N 10

FILE *Fp;
char Filed[NFILENAME];
char *Cont0[] = { "** Geodetic system adopted. **", "** Geocentric system adopted. **" };
char *Cont1[] = { "\nYear     altitude\n%8.3f %6.1f\n", "\nYear      radius\n%8.3f %7.1f\n" };
char *Cont1y[] = { "\nYear Mo Dy altitude\n%4d %2d %2d %6.1f\n", "\nYear Mo Dy  radius\n%4d %2d %2d %7.1f\n" };
char *Cont2 = "Latitude Longitude\n%6.2f    %6.2f\n";
char *Cont2d = "Lat-d Lat-m Lon-d Lon-m\n %3d    %2d  %4d    %2d\n";
char *Cont3 = "   X      Y      Z     F     I      D\n%6.0f %6.0f %6.0f %5.0f %6.2f %6.2f\n";
char *Cont4 = "   X      Y      Z     F     I      D\n   *      *   %6.0f %5.0f %6.2f   *\n\
(At the poles X, Y, D are indefinite. H=%6.0f)\n";
unsigned int Opt = 0;
int Gtype = GEODETIC, Yr, Mo, Dy, Latd, Latm, Lond, Lonm;
double Dyear, Htgd, Lat, Lon, X, Y, Z, I, D, F, **G, **H;

int main(argc, argv)
int argc;
char *argv[];
{
	int c;

	if (argc < 2){
		printf(HEAD);
		while (c=toupper(get_ch())){
			if (c == ZERO) break;
			else if (c == 'U') error(USAGE);
			else if (c == 'H') showless(HELP);
			else if (c == 'Q') exit(1);
			else continue;
		}
	}
	ig_cmd(argc,argv);
	G = mallocmd(N+1, N+1);   H = mallocmd(N+1, N+1);
	if (Opt & READFILE_BIT){
		ig_readf(N);
		freemd(N+1, G);  freemd(N+1, H);  fclose(Fp);
		exit(1);
	}
	printf("%s\n", Cont0[Gtype]);
	while (ig_year(&Dyear,&Htgd) != -1){
		if ((gausscoeff(Dyear,N,G,H)) == -1) error("Error in gausscoeff!");
		while (ig_site(&Lat,&Lon) != -1){
			gaussfld(Gtype, Lat, Lon, Htgd, N, &X, &Y, &Z, G, H);
			carmag(X, Y, Z, &F, &I, &D);
			checklon(&D, 1);
			if (!(Opt & YMD_BIT))
				printf(Cont1[Gtype], Dyear, Htgd);
			else
				printf(Cont1y[Gtype], Yr, Mo, Dy, Htgd);
			if (!(Opt & DEGMIN_BIT))
				printf(Cont2, Lat, Lon);
			else
				printf(Cont2d, Latd, Latm, Lond, Lonm);
			if (Lat<LTNPOLE && Lat>LTSPOLE)
				printf(Cont3, X, Y, Z, F, I, D);
			else
				printf(Cont4, Z, F, I, sqrt(X*X+Y*Y));
		}
	}
	freemd(N+1, G);   freemd(N+1, H);
}
#undef N
