/* pc_bing -- Bingham statistics of unit vectors   0919
		input N data of (inc,dec) or (lat,lon) where 3<=N<=50
*/

#include <stdlib.h>
#include "pcalc.h"

#define HEAD0 "Bingham statistics of unit vectors.\n\
  ** Depending on data, the MLE calculation might diverge! **\n\
  ** Although the process stops when iteration reaches 200,**\n\
  ** terminate it by Ctrl+C/kill command if it has frozen. **\n\
Continue? (Rtn or Y/N)"
#define HEAD1 "Input (Inc,Dec) or (Lat,Lon) (3<=N<=50):\n\
  (Rtn--end, Q--cancel)"
#define HEAD2 " i   I(Lat)  D(Lon)\n"
#define CONT2 "%2d  %6.2f  %6.2f\n"
#define CONT3 "Bingham statistics:\n  n     k1     k2     tau1   tau2   tau3\n\
 %2d  %6.2f %6.2f %6.2f %6.2f %6.2f\n"
#define CONT4 "  I3(Lat3) D3(Lon3)  I2(Lat2) D2(Lon2)  I1(Lat1) D1(Lon1)\n\
    %6.2f  %6.2f    %6.2f  %6.2f    %6.2f  %6.2f\n"
#define CONT5 "  a31    a32    a21     Xu     Xcp     Xcg\n\
 %5.2f  %5.2f  %5.2f  %6.2f  %6.2f  %6.2f\n"
#define CONT6 "Fisher statistics:\n  n Im(Latm) Dm(Lonm)   R       k     a95    Asd\n\
 %2d  %6.2f  %6.2f  %6.2f %7.2f %6.2f %6.2f\n"

extern FILE *Fp;
extern char Filed[];
extern int Optl;
extern double Ir, Dr, Km, R, A95, Asd, *Is, *Ds;

static int c, i, n;
static double k1, k2, tau1, tau2, inct[3], dect[3], aij[3], Xu, Xcp, Xcg;

void pc_bing()
{
	char rem[25];

	fprintf(stderr, "%s", HEAD0);
	if (yorn("", +1) == ZERO) return;

	printf("%s\n", HEAD1);
	while (1){
		i = 0;
		sprintf(rem, "I,D(Lt,Ln)_%d?", i+1);
		while ((c=inpd(rem,2,&Is[i],&Ds[i])) != -1){
			if (c == ZERO) return;
			i++;
			sprintf(rem, "I,D(Lt,Ln)_%d?", i+1);
			if (i >= NMAX) break;
		}
		n = i;
		if (n < 3) return;
		printf(HEAD2);
		for (i=0; i<n; i++) printf(CONT2, i+1, Is[i], Ds[i]);
		statf2(Is, Ds, n, &Ir, &Dr, &R, &Km, &A95, &Asd);
		cal_bing();
		if (!yorn("Re-run? (N or Rtn/Y)", -1)) break;
	}
}

/* cal_bing */

void cal_bing()
{
	if (statb(Is, Ds, n, &k1, &k2, &tau1, &tau2, inct, dect, aij) == ZERO)
		error("Iteration exceeded the maximum!");
	chitest(n, tau1, tau2, k1, k2, &Xu, &Xcp, &Xcg);
	printf(CONT3, n, k1, k2, tau1, tau2, (double)n-tau1-tau2);
	printf(CONT4, inct[2], dect[2], inct[1], dect[1], inct[0], dect[0]);
	printf(CONT5, aij[0], aij[1], aij[2], Xu, Xcp, Xcg);
	printf(CONT6, n, Ir, Dr, R, Km, A95, Asd);
}

/* pc_bing2 -- Bingham statistics from a file (N>=3) */

void pc_bing2()
{
	char s1[30], s2[30];

	fprintf(stderr, "%s", HEAD0);
	if (yorn("", +1) == ZERO) return;

	i = 0;
	while ((c=get_s(Fp,2,s1,s2)) != EOF){
		if (c==0 || s1[0]=='#' || s1[0]=='%' || s1[0]=='$') continue;
		i++;
	}
	n = i;
	Is = mallocvd(n);  Ds = mallocvd(n);
	rewind(Fp);

	i = 0;
	while ((c=get_s(Fp,2,s1,s2)) != EOF){
		if (c==0 || s1[0]=='#' || s1[0]=='%' || s1[0]=='$') continue;
		Is[i] = atof(s1);   Ds[i] = atof(s2);
		i++;
	}
	if (n != i) error("Error of count data!");
	if (n < 3) return;
	printf("Input file of Bingham statistics: %s\n", Filed);
	statf2(Is, Ds, n, &Ir, &Dr, &R, &Km, &A95, &Asd);
	cal_bing();
	if (Optl == 1){
		printf("Input data:\n");
		printf(HEAD2);
		for (i=0; i<n; i++) printf(CONT2, i+1, Is[i], Ds[i]);
	}
	freevd(Is);  freevd(Ds);
}
#undef HEAD0
#undef HEAD1
#undef HEAD2
#undef CONT2
#undef CONT3
#undef CONT4
#undef CONT5
#undef CONT6
