/* pc_circ -- circle fitting by Taubin method (data number <= NMAX=50).   1019 */

#include <string.h>
#include <stdlib.h>
#include <math.h>
#include "pcalc.h"

#define HEAD1 " i     X         Y         Error\n"
#define CONT1 "%2d %1s %#- 9.4g %#- 9.4g %#- 9.4g\n"
#define CONT2 " N   X0        Y0        R         k         SSE\n\
%2d %#- 9.4g %#- 9.4g %#- 9.4g %#- 9.4g %#- 9.4g\n"
#define HEAD2 " i   X         Y         Error\n"
#define CONT3 "%2d %#- 9.4g %#- 9.4g %#- 9.4g\n"

extern FILE *Fp;
extern char Filed[], **Rjct;
extern int Optl;
extern double *X, *Y, *Er, *Xt, *Yt;

static int c, i, j, k;
static double xc, yc, rc, kc, xm, ym, sse;

void pc_circ()
{
	char rem[20];
	int n, m;

	printf("Input X-Y data (3<=N<=50)\n");
	while (1){
		i = 0;
		sprintf(rem, "X,Y_%d?", i+1);
		while ((c=inpd(rem,2,&X[i],&Y[i])) != -1){
			if (c == ZERO) return;
			strcpy(Rjct[i], " ");
			i++;
			sprintf(rem, "X,Y_%d?", i+1);
			if (i >= NMAX) break;
		}
		n = i;
		if (n < 3) return;
		circreg(n, X, Y, &xc, &yc, &rc, &xm, &ym, Er, &sse);
		pc_cal_kc();
		m = n;
		pc_circ_rcal(n, m);
		if (!yorn("Re-run? (N or Rtn/Y)", -1)) break;
	}
}

/* pc_circ_rcal -- re-calculation of the circle after excluding one user selected */

void pc_circ_rcal(n, m)
int n, m;
{
	while (1){
		printf(HEAD1);
		for (i=0; i<n; i++) printf(CONT1, i+1, Rjct[i], X[i], Y[i], Er[i]);
		printf(CONT2, m, xc, yc, rc, kc, sse);
		if (m<4 || !yorn("Exclude one? (N or Rtn/Y)", -1)) break;
		else {
			while ((c=inpi("i?",1,&j)) != ZERO){
				if (c == -1) continue;
				if (j<1 || j>n) continue;
				strcpy(Rjct[j-1], "*");
				break;
			}
		}
		for (i=0,k=0; i<n; i++){
			if (strcmp(Rjct[i],"*") != ZERO){
				Xt[k] = X[i];  Yt[k] = Y[i];
				k++;
			}
		}
		m = k;
		circreg(m, Xt, Yt, &xc, &yc, &rc, &xm, &ym, Er, &sse);
		pc_cal_kc();
		pc_cal_er(n);
	}
}

/* pc_cal_kc */

void pc_cal_kc()
{
	if (xc==xm && yc==ym) kc = 0.;
	else if ((xc>xm && yc>ym) || (xc<xm && yc>ym)) kc = 1./rc;
	else if ((xc<xm && yc<ym) || (xc>xm && yc<ym)) kc = -1./rc;
	else {
		kc = 1./rc;
		fprintf(stderr, "** Strange circle center! **\n");
	}
}

/* pc_cal_er() */

void pc_cal_er(n)
int n;
{
	for (i=0; i<n; i++)
		Er[i] = sqrt((X[i]-xc)*(X[i]-xc) + (Y[i]-yc)*(Y[i]-yc)) - rc;
}

/* pc_circ2 -- circle fitting from a file (N>=3) */

void pc_circ2()
{
	char s1[30], s2[30];
	int n;

	i = 0;
	while ((c=get_s(Fp,2,s1,s2)) != EOF){
		if (c==0 || s1[0]=='#' || s1[0]=='%') continue;
		i++;
	}
	n = i;
	X = mallocvd(n);  Y = mallocvd(n);  Er = mallocvd(n);
	rewind(Fp);
	i = 0;
	while ((c=get_s(Fp,2,s1,s2)) != EOF){
		if (c==0 || s1[0]=='#' || s1[0]=='%') continue;
		X[i] = atof(s1);   Y[i] = atof(s2);
		i++;
	}
	if (n != i) error("Error of count data!");
	if (n < 3) return;
	printf("Input file of circle fitting: %s\n", Filed);
	circreg(n, X, Y, &xc, &yc, &rc, &xm, &ym, Er, &sse);
	pc_cal_kc();
	printf(CONT2, n, xc, yc, rc, kc, sse);
	if (Optl == 1){
		printf("Input data:\n");
		printf(HEAD2);
		for (i=0; i<n; i++) printf(CONT3, i+1, X[i], Y[i], Er[i]);
	}
	freevd(X);  freevd(Y);  freevd(Er);
}
#undef HEAD1
#undef CONT1
#undef CONT2
#undef HEAD2
#undef CONT3
