/* pc_meanuvec -- Fisher statistics of unit vectors (data number <= NMAX=50).
	type = FIELD:
		(I,D) are supposed for the data. Site location (Slat,Slon) is asked, and
		the mean vector is transferred to a corresponding VGP.
	type = VGP:
		(Plat,Plon) are supposed for the data, and hence the mean vector represents
		the grand-mean of the VGPs.
*/

#include <string.h>
#include "pcalc.h"

extern char **Rjct;
extern double Slt, Sln, Plt, Pln, Ir, Dr, Km, R, A95, Dp, Dm, Asd, *Is, *Ds, *It, *Dt, *Dev;

void pc_meanuvec(type)
int type;
{
	char cnt1[30], rem[25];
	int c, i, n, m;

	if (type == FIELD) strcpy(cnt1, "I,D_%d?");
	else strcpy(cnt1, "Plat,Plon_%d?");

	if (type == FIELD){
		if ((c=inpd("Slat,Slon? (Rtn to skip)",2,&Slt,&Sln)) == -1){
			Slt = 0.;   Sln = 0.;
		}
		else if (c == ZERO) return;
	}
	while (1){
		i = 0;
		sprintf(rem, cnt1, i+1);
		while ((c=inpd(rem,2,&Is[i],&Ds[i])) != -1){
			if (c == ZERO) return;
			It[i] = Is[i];   Dt[i] = Ds[i];
			strcpy(Rjct[i], " ");
			i++;
			sprintf(rem, cnt1, i+1);
			if (i >= NMAX) break;
		}
		n = i;
		if (n < 1) return;
		statf2(Is, Ds, n, &Ir, &Dr, &R, &Km, &A95, &Asd);
		m = n;
		pc_muvec_rcal(type, n, m);
		if (!yorn("Re-run? (N or Rtn/Y)", -1)) break;
	}
}

/* pc_muvec_rcal -- re-calculates the Fisher statistics after asking the user
	to exclude one datum
*/

void pc_muvec_rcal(type, n, m)
int type, n, m;
{
	char cnt2[50], cnt3[300];
	char *cnt2a = "%2d %1s %6.2f %6.2f %6.2f\n";
	int c, i, j, k;

	if (type == FIELD){
		strcpy(cnt2, " i      I      D     dev\n");
		strcpy(cnt3, " Slat   Slon   n   I      D       R      k      a95   Plat   Plon    dp     dm     asd\n");
		strcat(cnt3, "%6.2f %6.2f %2d %6.2f %6.2f %6.3f %7.2f %6.2f %6.2f %6.2f %6.2f %6.2f %6.2f\n");
	}
	else {
		strcpy(cnt2, " i    Plat   Plon    dev\n");
		strcpy(cnt3, " n  Plat   Plon     R      k      A95    Asd\n");
		strcat(cnt3, "%2d %6.2f %6.2f %6.3f %7.2f %6.2f %6.2f\n");
	}

	while (1){
		anglesd(Ir, Dr, n, Is, Ds, Dev);
		printf("%s", cnt2);
		for (i=0; i<n; i++) printf(cnt2a, i+1, Rjct[i], Is[i], Ds[i], Dev[i]);
		if (type == FIELD){
			fldvgp2(Slt, Sln, Ir, Dr, A95, &Plt, &Pln, &Dp, &Dm);
			printf(cnt3, Slt, Sln, m, Ir, Dr, R, Km, A95, Plt, Pln, Dp, Dm, Asd);
		}
		else
			printf(cnt3, m, Ir, Dr, R, Km, A95, Asd);
		if (m<3 || !yorn("Exclude one? (N or Rtn/Y)", -1)) break;
		else {
			while ((c=inpi("i?",1,&j)) != ZERO){
				if (c == -1) continue;
				if (j<1 || j>n) continue;
				strcpy(Rjct[j-1], "*");
				break;
			}
		}
		for (i=0,k=0; i<n; i++){
			if (strcmp(Rjct[i],"*") != ZERO){
				It[k] = Is[i];   Dt[k] = Ds[i];   k++;
			}
		}
		m = k;
		statf2(It, Dt, m, &Ir, &Dr, &R, &Km, &A95, &Asd);
	}
}
