/* pcalc -- assorted miscellaneous calculations in paleomagnetism
	V.3.6 -- update to meet the new gaussfld()  0420
	V.3.5 -- update to meet the new gausscoeff()   0320
	V.3.4 -- update for IGRF 2020 (IGRF-13)   0320
	V.3.3 -- circle fitting included   1019
	V.3.2 -- Bingham statistics included   0919
	V.3.1 -- major revision to pmag   0519
*/

#include "pcalc.h"

FILE *Fp;
char Filed[NFILENAME], **Rjct;
int Optl = 0;
double Slt, Sln, Plt, Pln, Ir, Dr, R, Km, A95, Dp, Dm, Asd, *Is, *Ds, *It, *Dt, *Dev, \
	*X, *Y, *Er, *Xt, *Yt;

int main(argc, argv)
int argc;
char *argv[];
{
	int c;

	c = pc_cmd(argc, argv);
	if ((Optl==1 && c==ASSORT) || (Optl==1 && c==IGRF))
		fprintf(stderr, "** -L option is ignored! **\n");
	if (c == ASSORT) pc_assort();
	else if (c == FIELD) pc_meanuvec2(FIELD);
	else if (c == VGP) pc_meanuvec2(VGP);
	else if (c == INCONLY) pc_meaninc2();
	else if (c == IGRF) pc_mag_igrf2();
	else if (c == BING) pc_bing2();
	else if (c == CIRC) pc_circ2();
	else error("Unknown option at the main!");
}

/* pc_assort -- assorted interactive calculations via the display */

void pc_assort()
{
	int c;

	Rjct = mallocms(NMAX,2);
	Is = mallocvd(NMAX);  Ds = mallocvd(NMAX);
	It = mallocvd(NMAX);  Dt = mallocvd(NMAX);  Dev = mallocvd(NMAX);
	X = mallocvd(NMAX);  Y = mallocvd(NMAX);  Er = mallocvd(NMAX);
	Xt = mallocvd(NMAX);  Yt = mallocvd(NMAX);
	while ((c=pc_assort_cmd(ITEMS)) != ZERO){
		switch(c){
		case '1':
			xyz_mid();
			break;
		case '2':
			mid_xyz();
			break;
		case '3':
			fld_vgp();
			break;
		case '4':
			vgp_fld();
			break;
		case '5':
			f_vdm();
			break;
		case '6':
			f_vadm();
			break;
		case '7':
			printf("Input data (N<=50): (Rtn--end, Q--cancel)\n");
			pc_meanuvec(FIELD);
			break;
		case '8':
			printf("Input data (N<=50): (Rtn--end, Q--cancel)\n");
			pc_meanuvec(VGP);
			break;
		case '9':
			printf("Input data (N<=50): (Rtn--end, Q--cancel)\n");
			pc_meaninc();
			break;
		case 'A':
			pc_mag_igrf();
			break;
		case 'B':
			pc_bing();
			break;
		case 'C':
			pc_circ();
			break;
		default:
			break;
		}
	}
	freems(NMAX, Rjct);
	freevd(Is);  freevd(Ds);  freevd(It);  freevd(Dt);  freevd(Dev);
	freevd(X);  freevd(Y);  freevd(Er), freevd(Xt);  freevd(Yt);
}
