/* sp2m -- Convert Natsuhara's SPIN2 files to those for "pdemag" and "pdirec".
	V2.1 -- minor amendments   0619
	V2.0 -- for new style files (*.txt) of Natsuhara's spinner 2000
		at Marine Core Center of Kochi University   0315,0812
	V1.2 -- for Natsuhara's old spinner   0903
	Usage:
	    sp2m [-C -S [-T] -N -H] file1 file2 ...
	Options:
	    C -- core coordinates data are taken
	    S -- in situ coordinates data
	   [T]-- bedding corrected data (default)
	    N -- for NRM measurements with output format for "pdirec"
	    H -- shows a help message
	Input files:
	    Natsuhara's new formatted file (*.txt) of SPIN2, which are
	    either of step-wise demagnetization data from a single sample,
	    or NRM measurements of multiple samples from a single site.
	    Reads <=50 files but use of a wild card "*" is NOT supported.
	Output files:
	    Results are written to stdout with either of the following formats.
	      Default, C, S options (for use by "pdemag"):
		$ file 0.0 0.0 0.0 0.0 (orientation info is set to zero)
		  step M I D             (coordinates depends on the option)
		    ...
		    ...
	      N option (for use by "pdirec")
		$ file
		  sample Ic Dc Is Ds It Dt -1.0 A 1 M ("-1.0 A 1" are dummies)\n\
		    ...
		    ...
	Basic idea:
		The program supposes that the input data are either of (1) series
	    of step-wise demagnetization from a single sample (-C,-S,[-T] options)
	    or (2) NRM measurements of multiple samples from a single site (-N
	    option). In the former, remanence directions in the specified
	    coordinates are taken so that the output data are used by "pdemag".
	    In the latter, directions in all coordinates are taken so that the
	    output data are used by "pdirec".
		Note that orientation and bedding attitude data are not included
	    in Natsuhara's new format. Hence, the best way to create the output
	    data to be used by "pdemag" would be that you take the directions in
	    core coordinates and, rewrite the orientation and bedding data at
	    the "$ line" in which now "0.0" is set. Alternatively, you may
	    convert the old format files (_*.txt) by "_sp2m". In this case,
	    orientation and bedding data are always included in the output data.
*/

#include "sp2m.h"

char Files[NFILES][LFILES];
unsigned Opt = 0;
int Frame = TILT;

int main(argc, argv)
int argc;
char *argv[];
{
	char *frm_lbl[] = {"CORE", "IN SITU", "BED CORRECTED"};
	int nf, i;

	if ((nf = sp2m_cmd(argc, argv)) < 1) error(USAGE);
	if ( !(Opt & NRM_BIT) ){
		fprintf(stdout, "# %s\n", frm_lbl[Frame]);
		for (i = 0; i < nf; i++) sp2m_cal(i);
	}
	else
		for (i = 0; i < nf; i++) sp2m_caln(i);
}
