#include <math.h>
#include "std.h"

/* anglesd -- ASD around (incr,decr), not necessarily mean, and angle distances   070199 */

double anglesd(incr, decr, n, inc, dec, dev)
int n;
double incr, decr, *inc, *dec, *dev;
{
	int i;
	double cosi, sini, cosd, sd2 = 0., d1;

	if (n < 1) error("wrong n in anglesd");
	cosi = cos(incr/RAD);   sini = sin(incr/RAD);
	for (i = 0; i < n; i++){
		d1 = inc[i]/RAD;
		cosd = cos(d1)*cosi*cos((dec[i] - decr)/RAD) + sin(d1)*sini;
		dev[i] = acos(cosd)*RAD;
		sd2 += dev[i]*dev[i];
	}
	if (n == 1)   return (0.);
	sd2 /= n-1;
	return sqrt(sd2);
}


/* anglesd2 -- ASD around (incr,decr), not necessarily mean, and angle distances   070199
		Within-site dispersion is NOT included, which should be handled by main
	Ver.2 -- Upper & lower 95% confidence limits were added.  However, it will not
		 be much useful when within-site dispersion is calculated because they
		 should be re-calculated for the new between-site dispersion   122200
		g: assign 1 for MOST CASES when true mean is NOT known
		   assign 0 ONLY when true mean is known for young rocks
*/

double anglesd2(incr, decr, n, inc, dec, dev, g, asdu, asdl)
int n, g;
double incr, decr, *inc, *dec, *dev, *asdu, *asdl;
{
	int i;
	double cosi, sini, cosd, sd2 = 0., d1;

	if (n < 1) error("wrong n in anglesd");
	cosi = cos(incr/RAD);   sini = sin(incr/RAD);
	for (i = 0; i < n; i++){
		d1 = inc[i]/RAD;
		cosd = cos(d1)*cosi*cos((dec[i] - decr)/RAD) + sin(d1)*sini;
		dev[i] = acos(cosd)*RAD;
		sd2 += dev[i]*dev[i];
	}
	if (n == 1)   return (0.);
	sd2 /= n-1;
	sd2 = sqrt(sd2);
	*asdu = sd2/sqrt(confcoxl(n, g, .05));
	*asdl = sd2/sqrt(confcoxu(n, g, .05));
	return(sd2);
}


/* confcoxu -- returns upper confidence limit for k of Cox (1969)   122200
	IMPORTANT NOTE:
	    Table 1 & 2 of Cox (1969) include error!
	    1st column should be N0+1, NOT N0-1.  Allan Cox made a simple mistake!
	Args:
	    n : number of vectors (N or N0 of Cox's notation)
	    alph: level of significance in decimal
		  (0.05 for 5% significance level or 95% confidence limit)
	    g : 0, when true mean is known (young volcanic rocks)
		1, when true mean is NOT known (USE THIS VALUE FOR MOST CASES)
	Variables:
	    p: probability for which Chi2(p) is returned
	    f: degree of freedom
		f = 2n, when true mean is known
		f = 2(n-1) when true mean is NOT known
*/

double confcoxu(n, g, alph)
int n, g;
double alph;
{
	int f;
	double p;

	if (n <= 0) error("Error of n at confcoxu!");
	if (alph > 0.5) error("Error of alph at confcoxu!");
	if (g < 0 || g > 1) error("Error of g at confcoxu!");
	p = alph/2.;
	f = 2*(n - g);
	return chi2tab(f, p)/f;
}


/* confcoxl -- returns lower confidence limit for k of Cox (1969) */

double confcoxl(n, g, alph)
int n, g;
double alph;
{
	int f;
	double p;

	if (n <= 0) error("Error of n at confcoxl!");
	if (alph > 0.5) error("Error of alph at confcoxl!");
	if (g < 0 || g > 1) error("Error of g at confcoxl!");
	p = 1. - alph/2.;
	f = 2*(n - g);
	return chi2tab(f, p)/f;
}


/* asdr_f2p -- gives Allan Cox's ratio of Sp/Sf at a lat when field is Fisherian   121600
		Sp^2/Sf^2 = (5 + 3*sin^2 t)(1 + 3*sin^2 t)/8 -- inverse of (12)
*/

double asdr_f2p(lat)
double lat;
{
	double sint, sint2;

	sint2 = (sint = sin(lat/RAD))*sint;
	return sqrt( (5. + 3.*sint2)*(1. + 3.*sint2)/8. );
}


/* asdr_p2f -- gives Allan Cox's ratio of Sf/Sp at a lat when pole is Fishereian
		Sf^2/Sp^2 = (5 + 3*sin^2 t)/(2*(1 + 3*sin^2 t)^2) -- inverse of (24)
*/

double asdr_p2f(lat)
double lat;
{
	double sint, sint2, dumm;

	sint2 = (sint = sin(lat/RAD))*sint;
	dumm = 1. + 3.*sint2;
	return sqrt( (5. + 3.*sint2)/(2.*dumm*dumm) );
}
