#include <math.h>
#include "std.h"

#define EPS 1.e-10

/* carmag -- conversion from cartesian to geomagnetic coordinate  12/96
	minor amendment to the usage of atan & atan2  0619
*/

void carmag(x, y, z, mag, inc, dec)
double x, y, z, *mag, *inc, *dec;
{
	double xa, ya, h;

	*mag = sqrt(x*x + y*y + z*z);
	if ((h=sqrt(x*x+y*y))<EPS && z>=0.){
		*inc = 90.;  *dec = 0.;
		return;
	}
	else if (h<EPS && z<0.){
		*inc = -90.;  *dec = 0.;
		return;
	}
	else
		*inc = RAD*atan(z/h);
	*dec = RAD*atan2(y, x);
	checklon(dec, 0);
}

/* magcar */

void magcar(mag, inc, dec, x, y, z)
double mag, inc, dec, *x, *y, *z;
{
	double dumi, dumd, cosi, sini, cosd, sind;

	dumi = inc/RAD;   dumd = dec/RAD;
	cosi = cos(dumi);   sini = sin(dumi);
	cosd = cos(dumd);   sind = sin(dumd);
	*x = mag*cosi*cosd;
	*y = mag*cosi*sind;
	*z = mag*sini;
}

/* carsph -- cartesin to spherical polar coordinates   092800 */

void carsph(x, y, z, r, th, phi)
double x, y, z, *r, *th, *phi;
{
	double inc;

	carmag(x, y, z, r, &inc, phi);
	*th = 90. - inc;
}

/* sphcar */

void sphcar(r, th, phi, x, y, z)
double r, th, phi, *x, *y, *z;
{
	double inc;

	inc = 90. - th;
	magcar(r, inc, phi, x, y, z);
}
#undef EPS
