/* gammln.c -- Gamma Function and others from Section 6.1 of Numerical Recipes 
	by Press et al.1992, modified by Hidefumi   111898 */

#include <math.h>
#include "std.h"

/* gammln -- ln(gamma(xx)) */

double gammln(double xx)
{
	double x,y,tmp,ser;
	static double cof[6]={76.18009172947146,-86.50532032941677,
		24.01409824083091,-1.231739572450155,
		0.1208650973866179e-2,-0.5395239384953e-5};
	int j;

	y=x=xx;
	tmp=x+5.5;
	tmp -= (x+0.5)*log(tmp);
	ser=1.000000000190015;
	for (j=0;j<=5;j++) ser += cof[j]/++y;
	return -tmp+log(2.5066282746310005*ser/x);
}


/* factrl -- n! as a double */

double factrl(int n)
{
	static int ntop=4;
	static double a[33]={1.0,1.0,2.0,6.0,24.0};
	int j;

	if (n < 0) error("Negative factorial in routine factrl!");
	if (n > 32) return exp(gammln(n+1.0));
	while (ntop<n) {
		j=ntop++;
		a[ntop]=a[j]*ntop;
	}
	return a[n];
}


/* bico -- binomial coefficient n!/k!(n-k)! */

double bico(int n, int k)
{
	return floor(0.5+exp(factln(n)-factln(k)-factln(n-k)));
}


/* factln -- ln(n!) */

double factln(int n)
{
	static double a[101];

	if (n < 0) error("Negative factorial in routine factln!");
	if (n <= 1) return 0.0;
	if (n <= 100) return a[n] ? a[n] : (a[n]=gammln(n+1.0));
	else return gammln(n+1.0);
}


/* beta -- beta function B(z,w) */

double beta(double z, double w)
{
	return exp(gammln(z)+gammln(w)-gammln(z+w));
}
