/* lgndr --  Schmidt half-normalized Legendre function   2/20/96, 6/14/86 
	n <= 8: explicit formula modified from Iwanami III, p.82, p.131
	n > 8: call recurrence formula plgndr() adapted from Numerical Recepies
*/

#include <math.h>
#include "std.h"

double lgndr(n, m, x)
int n, m;
double x; 
{
	int k;
	double y, x2, ym, y2, y3, d;

	x2 = x*x;
	y2 = 1. - x2;
	y = sqrt(y2);
	if (n<0 || m<0)
		error("lgndr: the order or dgree is invalid");
	else if (n < m)
		return(0.);
	else if (n == 0)
		return(1.);
	else if (n == 1){
		if (m == 0) return(x);
		else if (m == 1) return(y); }
	else if (n == 2){
		if (m==0) return( 0.5*(3.*x2-1.) );
		else if (m==1) return( 1.7320508*y*x );  /* sqrt(3) */
		else if (m==2) return( 0.86602540*y*y ); }  /* sqrt(3)/2 */
	else if (n == 3){
		if (m==0) return( 0.5*x*(5.*x2-3.) );
		else if (m==1) return(0.61237244*y*(5.*x2-1.));  /* sqrt(6)/4 */
		else if (m==2) return( 1.9364917*y2*x );  /* sqrt(15)/2 */
		else if (m==3) return( 0.79056942*y2*y ); }  /* sqrt(10)/4 */
	else if (n == 4){
		if (m==0) return( 0.125*(35.*x2*x2-30.*x2+3.) );
		else if (m==1) return( 0.79056942*y*x*(7.*x2-3.) );  /* sqrt(10)/4 */
		else if (m==2) return(0.55901699*y2*(7.*x2-1.));  /* sqrt(5)/4 */
		else if (m==3) return( 2.0916501*y2*y*x );  /* sqrt(70)/4 */
		else if (m==4) return( 0.73950997*y2*y2 ); }  /* sqrt(35)/8 */
	else if (n == 5){
		if (m==0) return( 0.125*x*(63.*x2*x2-70.*x2+15.) );
		else if (m==1) return( 0.48412292*y*(21.*x2*x2-14.*x2+1.) );  /* sqrt(15)/8 */
		else if (m==2) return( 2.5617377*y2*x*(3.*x2-1.) );  /* sqrt(105)/4 */
		else if (m==3) return( 0.52291252*y2*y*(9.*x2-1.) );  /* sqrt(70)/16 */
		else if (m==4) return( 2.2185299*y2*y2*x );  /* 3*sqrt(35)/8 */
		else if (m==5) return(0.70156076*y2*y2*y); }  /* 3*sqrt(14)/16 */
	else if (n == 6){
		if (m==0) return( 0.0625*(231.*x2*x2*x2-315.*x2*x2+105.*x2-5.) );
		else if (m==1) return( 0.57282196*y*x*(33.*x2*x2-30.*x2+5.) );  /* sqrt(21)/8 */
		else if (m==2) return( 0.45285552*y2*(33.*x2*x2-18.*x2+1.) );  /* sqrt(210)/32 */
		else if (m==3) return( 0.90571105*y2*y*x*(11.*x2-3.) );  /* sqrt(210)/16 */
		else if (m==4) return( 0.49607837*y2*y2*(11.*x2-1.) );  /* 3*sqrt(7)/16 */
		else if (m==5) return( 2.3268138*y2*y2*y*x );  /* 3*sqrt(154)/16 */
		else if (m==6) return( 0.67169329*y2*y2*y2 ); }  /* sqrt(462)/32 */
	else if (n == 7){
		if (m==0) return( 0.0625*x*(429.*x2*x2*x2-693.*x2*x2+315.*x2-35.) );
		else if (m==1) return( 0.082679728*y*(429.*x2*x2*x2-495.*x2*x2+135.*x2-5.) );  /* sqrt(7)/32 */
		else if (m==2) return( 0.20252315*y2*x*(143.*x2*x2-110.*x2+15.) );  /* sqrt(42)/32 */
		else if (m==3) return( 0.14320549*y2*y*(143.*x2*x2-66.*x2+3.) );  /* sqrt(21)/32 */
		else if (m==4) return( 0.94991776*y2*y2*x*(13.*x2-3.) );  /* sqrt(231)/16 */
		else if (m==5) return( 0.47495888*y2*y2*y*(13.*x2-1.) );  /* sqrt(231)/32 */
		else if (m==6) return( 2.4218246*y2*y2*y2*x );  /* sqrt(6006)/32 */
		else if (m==7) return( 0.64725985*y2*y2*y2*y ); }  /* sqrt(429)/32 */
	else if (n == 8){
	    if (m==0) return( 0.0078125*(6435.*x2*x2*x2*x2-12012.*x2*x2*x2+6930.*x2*x2-1260.*x2+35.) );
	    else if (m==1) return( 0.09375*y*x*(715.*x2*x2*x2-1001.*x2*x2+385.*x2-35.) );  /* 3/32 */
	    else if (m==2) return( 0.39218439*y2*(143.*x2*x2*x2-143.*x2*x2+33.*x2-1.) );  /* 3*sqrt(70)/64 */
	    else if (m==3) return( 1.0620403*y2*y*x*(39.*x2*x2-26.*x2+3.) );  /* sqrt(1155)/32 */
	    else if (m==4) return( 0.41132646*y2*y2*(65.*x2*x2-26.*x2+1.) );  /* 3*sqrt(77)/64 */
	    else if (m==5) return( 2.9661173*y2*y2*y*x*(5.*x2-1.) );  /* 3*sqrt(1001)/32 */
	    else if (m==6) return( 0.45768183*y2*y2*y2*(15.*x2-1.) );  /* sqrt(858)/64 */
	    else if (m==7) return( 2.5068266*y2*y2*y2*y*x );  /* 3*sqrt(715)/32 */
	    else if (m==8) return( 0.62670665*y2*y2*y2*y2 ); }  /* 3*sqrt(715)/128 */
	else 
        	return plgndr(n, m, x);
}

/* plgndr -- Schmidt half-normalized, modified from p.254 of Press et al. (1992) */

double plgndr(l, m, x)
int l, m;
double x;
{
	double fact, pll, pmm, pmmp1, somx2, f2m;
	int i, ll;

	if (m<0 || m>l || fabs(x)>1.)
		error("Bad arguments in routine plgndr");
	pmm = 1.;
	if (m > 0) {
		somx2 = sqrt((1. - x)*(1. + x));
		fact = 1.;   f2m = 1.;
		for (i=1; i<=m; i++){
			pmm *= fact*somx2;
			f2m *= fact*(fact + 1.);
			fact += 2.;
		}
		pmm *= sqrt(2./f2m);
	}
	if (l == m)
		return pmm;
	else {
	    pmmp1 = x*sqrt((double)(2*m + 1))*pmm;
	    if (l == (m+1))
		return pmmp1;
	    else {
		for (ll=m+2; ll<=l; ll++){
		    pll = x*(2*ll-1)*pmmp1/sqrt((double)(ll*ll-m*m)) - sqrt( ((double)((ll-1)*(ll-1)-m*m))/((double)(ll*ll-m*m)) )*pmm;
		    pmm = pmmp1;
		    pmmp1 = pll;
		}
		return pll;
	    }
	}
}

/* dlgndr -- theta derivative of Schimdt normalized Legendre function   2/20/96
		dp(n,m,x)/dz = sqrt((n+m)*(n-m+1))*p(n,m-1,x)/2 - 
					sqrt((n+m+1)(n-m))*p(n,m+1,x)/2
		dp(n,1,x)/dz = sqrt(2*(n+1)*n)*p(n,0,x)/2 -
					sqrt((n+2)*(n-1))*p(n,2,x)/2
		dp(n,0,x)/dz = -sqrt(n*(n+1)/2)*p(n,1,x)
		where x = cos(z)
*/

double dlgndr(n, m, x)
int n, m;
double x;
{
	double nn, mm;

	nn = n;   mm = m;
	if (m == 0)
		return -sqrt(0.5*nn*(nn + 1.))*lgndr(n, 1, x);
	else if (m == 1)
		return 0.5*( sqrt(2.*(nn + 1.)*nn)*lgndr(n, 0, x) - sqrt((nn + 2.)*(nn - 1.))*lgndr(n, 2, x) );
	else
		return 0.5*( sqrt((nn + mm)*(nn - mm + 1.))*lgndr(n, m-1, x) - sqrt((nn + mm + 1.)*(nn - mm))*lgndr(n, m+1, x) );
}

/* mpsin -- m(Pnm)/sin(theta) to avoid singularity for Schmidt normalization (n<=8)   2/22/96
	added n>8 by a formula adapted from (39) at p.256 of Langel (1987)   093000
*/

double mpsin(n, m, x)
int n, m;
double x;
{
        int k;
        double y, x2, ym, y2, y3, d;

        x2 = x*x;
        y2 = 1. - x2;
        y = sqrt(y2);
        if (n<0 || m<0)
                error("Negative n or m at mpsin!");
        else if (n < m)
                return(0.);
        else if (n == 0)
                return(1.);
        else if (n == 1){
                if (m == 0) return(0.);
                else if (m == 1) return(1.); }
        else if (n == 2){
                if (m==0) return( 0. );
                else if (m==1) return( 1.7320508*x );  /* sqrt(3) */
                else if (m==2) return( 1.7320508*y ); }  /* sqrt(3) */
        else if (n == 3){
                if (m==0) return( 0. );
                else if (m==1) return(0.61237244*(5.*x2-1.));  /* sqrt(6)/4 */
                else if (m==2) return( 3.8729834*y*x );  /* sqrt(15) */
                else if (m==3) return( 2.3717082*y2 ); }  /* 3*sqrt(10)/4 */
        else if (n == 4){
                if (m==0) return( 0. );
                else if (m==1) return( 0.79056942*x*(7.*x2-3.) );  /* sqrt(10)/4 */
                else if (m==2) return(1.1180340*y*(7.*x2-1.));  /* sqrt(5)/2 */
                else if (m==3) return( 6.2749502*y2*x );  /* 3*sqrt(70)/4 */
                else if (m==4) return( 2.9580399*y2*y ); }  /* sqrt(35)/2 */
        else if (n == 5){
                if (m==0) return( 0. );
                else if (m==1) return( 0.48412292*(21.*x2*x2-14.*x2+1.) );  /* sqrt(15)/8 */
                else if (m==2) return( 5.1234754*y*x*(3.*x2-1.) );  /* sqrt(105)/2 */
                else if (m==3) return( 1.5687375*y2*(9.*x2-1.) );  /* 3*sqrt(70)/16 */
                else if (m==4) return( 8.8741197*y2*y*x );  /* 3*sqrt(35)/2 */
                else if (m==5) return(3.5078038*y2*y2); }  /* 15*sqrt(14)/16 */
        else if (n == 6){
                if (m==0) return( 0. );
                else if (m==1) return( 0.57282196*x*(33.*x2*x2-30.*x2+5.) );  /* sqrt(21)/8 */
                else if (m==2) return( 0.90571105*y*(33.*x2*x2-18.*x2+1.) );  /* sqrt(210)/16 */
                else if (m==3) return( 2.7171331*y2*x*(11.*x2-3.) );  /* 3*sqrt(210)/16 */
                else if (m==4) return( 1.9843135*y2*y*(11.*x2-1.) );  /* 3*sqrt(7)/4 */
                else if (m==5) return( 11.634069*y2*y2*x );  /* 15*sqrt(154)/16 */
                else if (m==6) return( 4.0301597*y2*y2*y ); }  /* 3*sqrt(462)/16 */
        else if (n == 7){
                if (m==0) return( 0. );
                else if (m==1) return( 0.082679728*(429.*x2*x2*x2-495.*x2*x2+135.*x2-5.) );  /* sqrt(7)/32 */
                else if (m==2) return( 0.40504629*y*x*(143.*x2*x2-110.*x2+15.) );  /* sqrt(42)/16 */
                else if (m==3) return( 0.42961647*y2*(143.*x2*x2-66.*x2+3.) );  /* 3*sqrt(21)/32 */
                else if (m==4) return( 3.7996710*y2*y*x*(13.*x2-3.) );  /* sqrt(231)/4 */
                else if (m==5) return( 2.3747944*y2*y2*(13.*x2-1.) );  /* 5*sqrt(231)/32 */
                else if (m==6) return( 14.530948*y2*y2*y*x );  /* 3*sqrt(6006)/16 */
                else if (m==7) return( 4.5308189*y2*y2*y2 ); }  /* 7*sqrt(429)/32 */
        else if (n == 8){
            if (m==0) return( 0. );
            else if (m==1) return( 0.09375*x*(715.*x2*x2*x2-1001.*x2*x2+385.*x2-35.) );  /* 3/32 */
            else if (m==2) return( 0.78436877*y*(143.*x2*x2*x2-143.*x2*x2+33.*x2-1.) );  /* 3*sqrt(70)/32 */
            else if (m==3) return( 3.1861210*y2*x*(39.*x2*x2-26.*x2+3.) );  /* 3*sqrt(1155)/32 */
            else if (m==4) return( 1.6453058*y2*y*(65.*x2*x2-26.*x2+1.) );  /* 3*sqrt(77)/16 */
            else if (m==5) return( 14.830586*y2*y2*x*(5.*x2-1.) );  /* 15*sqrt(1001)/32 */
            else if (m==6) return( 2.7460910*y2*y2*y*(15.*x2-1.) );  /* 3*sqrt(858)/32 */
            else if (m==7) return( 17.5477863*y2*y2*y2*x );  /* 21*sqrt(715)/32 */
            else if (m==8) return( 5.0136532*y2*y2*y2*y ); }  /* 3*sqrt(715)/16 */
        else {
		if (m == 0) return( 0. );
		if (m == 1) return( 0.5*(sqrt(2*(n+1)*n)*lgndr(n-1,0,x) + sqrt((n-1)*(n-2))*lgndr(n-1,2,x)) );
		else return( 0.5*(sqrt((n+m)*(n+m-1))*lgndr(n-1,m-1,x) + sqrt((n-m)*(n-m-1))*lgndr(n-1,m+1,x)) ); }
}
