#include <stdlib.h>
#include "std.h"

/* mallocmi -- memory allocation for integer matrix, or 2-D array, arranged from   092900
		Numerical Recipes Chapter 1.2 p.21 with suggestion by Hatakeyama
	Example of usage:
		(at main program, declare as,)
			int **A, **mallocmi();
		(and allocate memory for 3x4 matrix as,)
			A = mallocmi(3, 4);
		(use normally in the main or call subprog as,)
			subprog(3, 4, A, ...)
		(the subprog would look like as,)
			subprog(int n, int m, int **matrix, ...)
			  ...
			for(i = 0; i < n; i++)
				for (j = 0; j < m; j++)
					matrix[i][j] = ...
		(to free the allocated memory, do in the main as,)
			freemi(3, A);
*/

int **mallocmi(n, m)
int n, m;
{
	int i, **x;

	x = (int **) malloc( (size_t)(n*sizeof(int*)) );
	if (x == NULL) error("Error of malloc at mallocmi!");
	for (i = 0; i < n; i++){
		x[i] = (int *) malloc( (size_t)(m*sizeof(int)) );
		if (x[i] == NULL) error("Error of malloc at mallocmi!");
	}
	return(x);
}

/* mallocmd -- double matrix version */

double **mallocmd(n, m)
int n, m;
{
	int i;
	double **x;

	x = (double **) malloc( (size_t)(n*sizeof(double*)) );
	if (x == NULL) error("Error of malloc at mallocmd!");
	for (i = 0; i < n; i++){
		x[i] = (double *) malloc( (size_t)(m*sizeof(double)) );
		if (x[i] == NULL) error("Error of malloc at mallocmd!");
	}
	return(x);
}

/* mallocms -- memory alloc for character table, which is a 2-D array of char
	Example of usage:
		char **s, **mallocms();
		s = mallocms(3, 30);
		strcpy(s[0], "String of length less than 30 chars.");
		strcpy(s[1], "Second string comes here.");
		strcpy(s[2], "Third string is the last line.");
		  ...
		for (i = 0; i < 3; i++)
			printf("%1d: %s\n", i+1, s[i]);
		
		(the above code will show the next table)
		1: String of length less than 30 chars.
		2: Second string comes here.
		3: Third string is the last line.
*/
 
char **mallocms(n, m)
int n, m;
{
	int i;
	char **x;

	x = (char **) malloc( (size_t)(n*sizeof(char*)) );
	if (x == NULL) error("Error of malloc at mallocms!");
	for (i = 0; i < n; i++){
		x[i] = (char *) malloc( (size_t)(m*sizeof(char)) );
		if (x[i] == NULL) error("Error of malloc at mallocms!");
	}
	return(x);
}

/* freemi -- free the memory allocated by mallocmi
	NOTE: only one arg (n) is used because "m" is not necessary
	      pay attention to the usage freemi(n, A) vs. mallocmi(n, m)
*/

void freemi(n, x)
int n, **x;
{
	int i;

	for (i = n-1; i >= 0; i--)
		free(x[i]);
	free(x);
}

/* freemd -- free the double matrix */

void freemd(n, x)
int n;
double **x;
{
	int i;

	for (i = n-1; i >= 0; i--)
		free(x[i]);
	free(x);
}

/* freems -- free the character table */

void freems(n, x)
int n;
char **x;
{
	int i;

	for (i = n-1; i >= 0; i--)
		free(x[i]);
	free(x);
}
