#include <math.h>
#include <stdlib.h>
#include "std.h"

/* pcagtc -- principal component analysis of a great circle of unit vectors in which
	the fitted plane is anchored to the origin, modified from pcalin. V.1.0   082803
    Arguments:
	inc, dec -- nd sets of directions.  Ordinary nd-dimension array (0,..,nd-1)
	incp, decp -- pole for the GTC which is the eigen vector of the smallest eigen value
	mad -- maximum angular deviation for the pole (last eq. of Eq.3 in Kirschvink)
    Variables:
	T,V: T matrix and a matrix of eigen vectors in coluyzs.
	     4x4 matrixes, but use as (1,2,3)x(1,2,3)
	d: vector of eigen values.
	   4-dimension vector, but use as (1,2,3)
    Returns:
	ZERO when error in jacobi(), otherwise returns +1.
    Notes:
	mallocmd() does not work to T & V (end up with segmentation error)
*/

int pcagtc(inc, dec, nd, incp, decp, mad)
int nd;
double *inc, *dec, *incp, *decp, *mad;
{
	int i, nrot, i1, i2, i3;
	double *x, *y, *z, xx=0., xy=0., xz=0., yy=0., yz=0., zz=0., **T, d[4], **V, \
		ux, uy, uz, dumm;

	if (nd <= 1) error("pcagtc: error of nd <= 1!");
	x = mallocvd(nd);   y = mallocvd(nd);   z = mallocvd(nd);
        if ( (T = (double **) malloc( (size_t)(4*sizeof(double*)) ) )
                == NULL ) error("error in memory allocation of T");
        if ( (*T = (double *) malloc( (size_t)(16*sizeof(double)) ) )
                == NULL ) error("error in memory allocation of *T");
        if ( (V = (double **) malloc( (size_t)(4*sizeof(double*)) ) )
                == NULL ) error("error in memory allocation of V");
        if ( (*V = (double *) malloc( (size_t)(16*sizeof(double)) ) )
                == NULL ) error("error in memory allocation of *V");
/*
	T = mallocmd(4, 4);   V = mallocmd(4, 4);
*/
	for (i = 0; i < nd; i++)
		magcar(1., inc[i], dec[i], x+i, y+i, z+i);
	for (i=0; i<nd; i++){
		xx += x[i]*x[i];   xy += x[i]*y[i];   xz += x[i]*z[i];
		yy += y[i]*y[i];   yz += y[i]*z[i];   zz += z[i]*z[i];
	}
	*(*T+5) = xx;   *(*T+6) = xy;   *(*T+7) = xz;
	*(*T+9) = xy;   *(*T+10) = yy;   *(*T+11) = yz;
	*(*T+13) = xz;   *(*T+14) = yz;   *(*T+15) = zz;
	if (jacobi(T, 3, d, V, &nrot) == ZERO) return(ZERO);
	eigenorder(d, &i1, &i2, &i3);
	ux = *(*V+4+i1);   uy = *(*V+8+i1);   uz = *(*V+12+i1);
	carmag(ux, uy, uz, &dumm, incp, decp);
	if (nd > 2)
		*mad = RAD*atan( sqrt( d[i1]/d[i2] + d[i1]/d[i3] ) );
	else
		*mad = -1.;
	freevd(x);   freevd(y);   freevd(z);
	free(T);   free(V);
/*	freemd(4, T);   freemd(4, V); */
	return(1);
}
/*
memory of 4x4 matrix which is used to calculate 3x3 matrix
	 0   1   2   3
	 4   5   6   7
	 8   9  10  11
	12  13  14  15
*/

/* pcagtca -- automatic determination of the great circle of the highest component with
	alogrithm simplified from Kirshvink (1980). PCA analysis by pcagtc is made to
	unit vecotors, in which the fitted plane is anchored to the origin.   060918
    Arguments:
	madc -- critical MAD to be a planar points
	I1, I2 -- selected range of steps to cover the plane
    Returns:
	ZERO when no plane was found, otherwise returns +1.
    For other details, see the notes in pcagtc.
*/

int pcagtca(inc, dec, nd, madc, incp, decp, I1, I2, mad)
int nd, *I1, *I2;
double *inc, *dec, madc, *incp, *decp, *mad;
{
	int i1, i2, ni;
	double madi;

	if (nd < 3) error("pcagtca: error of nd < 4!");
	i1 = nd - 3;  i2 = nd - 1;  madi = 0.;
	ni = i2 - i1 + 1;
	pcagtc(inc+i1, dec+i1, ni, incp, decp, &madi);
/*	printf("Check-0:i1,i2,ni,madi: %d %d %d %.1f\n",i1,i2,ni,madi); */

/* shift a window */

	while (madi > madc){
		if (i1 >= 1){
			i1 -= 1;  i2 -= 1;
			ni = i2 - i1 + 1;
			pcagtc(inc+i1, dec+i1, ni, incp, decp, &madi);
/*			printf("Check-1:i1,i2,ni,madi: %d %d %d %.1f\n",i1,i2,ni,madi); */
			continue;
		}
		else {
/*			printf("Check-2:i1,i2,ni,madi: %d %d %d %.1f\n",i1,i2,ni,madi); */
			*I1 = i1;  *I2 = i2;  *mad = madi;
			return ZERO;  /* No planar three consecutive points */
		}
	}

/* expand i1 toward i1=0 */

	if (i1 >= 1){
		i1 -= 1;
		ni = i2 - i1 + 1;
		pcagtc(inc+i1, dec+i1, ni, incp, decp, &madi);
/*		printf("Check-3:i1,i2,ni,madi: %d %d %d %.1f\n",i1,i2,ni,madi); */
		while (madi <= madc){
			if (i1 >= 1){
				i1 -= 1;
				ni = i2 - i1 + 1;
				pcagtc(inc+i1, dec+i1, ni, incp, decp, &madi);
/*				printf("Check-4:i1,i2,ni,madi: %d %d %d %.1f\n",i1,i2,ni,madi); */
				continue;
			}
			else {
				i1 -= 1;  /* set i1=-1 so that later planar for i1=0 to i2 */
/*				printf("Check-5:i1,i2,ni,madi: %d %d %d %.1f\n",i1,i2,ni,madi); */
				break;
			}
		}
		i1 += 1;  /* Restore i1 */
/*		printf("Check-6:i1,i2,ni,madi: %d %d %d %.1f\n",i1,i2,ni,madi); */
	}

/* expand i2 toward i2=nd-1 */

	if (i2 <= nd-2){
		i2 += 1;
		ni = i2 - i1 + 1;
		pcagtc(inc+i1, dec+i1, ni, incp, decp, &madi);
/*		printf("Check-7:i1,i2,ni,madi: %d %d %d %.1f\n",i1,i2,ni,madi); */
		while (madi <= madc){
			if (i2 <= nd-2){
				i2 += 1;
				ni = i2 - i1 + 1;
				pcagtc(inc+i1, dec+i1, ni, incp, decp, &madi);
/*				printf("Check-8:i1,i2,ni,madi: %d %d %d %.1f\n",i1,i2,ni,madi); */
				continue;
			}
			else {
				i2 += 1;  /* set i2=nd so that later planar for i1 to i2=nd-1 */
/*				printf("Check-9:i1,i2,ni,madi: %d %d %d %.1f\n",i1,i2,ni,madi); */
				break;
			}
		}
		i2 -= 1;  /* Restore i2 */
/*		printf("Check-10:i1,i2,ni,madi: %d %d %d %.1f\n",i1,i2,ni,madi); */
	}

/* do pcagtc with final i1 & i2, and return linear results */

	ni = i2 - i1 + 1;
	pcagtc(inc+i1, dec+i1, ni, incp, decp, &madi);
/*	printf("Check-11:i1,i2,ni,madi: %d %d %d %.1f\n",i1,i2,ni,madi); */
	*I1 = i1;  *I2 = i2;  *mad = madi;
	return 1;
}
