#include <math.h>
#include "std.h"

#define EPS 1.0e-6
#define JMAX 20

/* qsimp -- returns integral of func from a to b by Simpson's rule   8/13/96
	modified from Press et al., Numerical Recepies, p.139, 1992.
	minor amendments in the dfined constants   0819
*/

double qsimp(func, a, b)
double (*func)(), a, b;
{
	int j;
	double s, st, ost, os;

	ost = os = -1.0e30;
	for (j = 1; j <= JMAX; j++){
		st = trapzd(func, a, b, j);
		s = (4.*st - ost)/3.;
		if ( fabs(s - os) < EPS*fabs(os) ) return s;
		os = s;
		ost = st;
	}
	error("Too many steps in routine qsimp");
	return 0.;
}

/* qtrap -- returns integral of func from a to b by trapezoidal rule   8/13/96
	modified from Press et al., Numerical Recepies, p.137, 1992.
*/

double qtrap(func, a, b)
double (*func)(), a, b;
{
	int j;
	double s, olds;

	olds = -1.0e30;
	for (j = 1; j <= JMAX; j++){
		s = trapzd(func, a, b, j);
		if ( fabs(s - olds) < EPS*fabs(olds) ) return s;
		olds = s;
	}
	error("Too many steps in routine qtrap");
	return 0.;
}

/* trapzd -- integration by extended trapezoidal rule modified from   8/12/96
	Press et al., Numerical Recepies in C, p.137, 1992.
	First call with n=1 returns crudest estimate of integral(a->b)
	Subsequent calls with n=2,3,... will improve by adding 2^(n-2)
	Never use to more than 1 functions at the same time!
*/

double trapzd(func, a, b, n)
int n;
double (*func)(), a, b;
{
	double x, tnm, sum, del;
	static double s;
	int it, j;

	if (n == 1)
		return ( s = 0.5*(b - a)*( (*func)(a) + (*func)(b) ) );
	else {
		for (it = 1, j = 1; j < n-1; j++) it <<= 1;
		tnm = it;
		del = (b - a)/tnm;
		x = a + 0.5*del;
		for (sum = 0.0, j = 1; j <= it; j++, x += del)
			sum += (*func)(x);
		s = 0.5*(s + (b - a)*sum/tnm);
		return s;
	}
}
#undef EPS
#undef JMAX
