/* ranf2 -- Fisher distributed random points recommended by McFadden & McElhinny (1990),
	which is after Fisher et al. (1981), or p.59 of the text book (Fisher et al. 1987).
	ranf was abandoned because of failure for small k <= ~2   091018
	theta & phi: 0-pi & 0-2pi in radian, repectively.
*/

#include <math.h>
#include "std.h"

void ranf2(k, theta, phi)
double k, *theta, *phi;
{
	static long idum[] = {-1};
	double lambda, dum;

	dum = exp(-2.*k);
	lambda = -log((1. - dum)*ran1(idum) + dum)/(2.*k);
	*theta = 2.*asin(sqrt(lambda));
	*phi = DPAI*ran1(idum);
}

/* ranb -- random points of Bingham distribution for 0<=theta<=pi/2   1296
	returns number of rejected points of theta>pi/2
	"*idum = -1" is introduced   0699
	altered to (theta,phi) in radian and ranf2() is used   0819
*/

int ranb(k1, k2, theta, phi)
double k1, k2, *theta, *phi;
{
	static long idum[] = {-1};
	int nsouth = 0;
	double k, lambda, bfr, cost, sint, cosp, sinp, dum;

	k = -k2;
	do {
		dum = exp(-2.*k);
		lambda = -log((1. - dum)*ran1(idum) + dum)/(2.*k);
		*theta = 2.*asin(sqrt(lambda));
		*phi = DPAI*ran1(idum);
		cost = cos(*theta);   sint = sin(*theta);
		cosp = cos(*phi);   sinp = sin(*phi);
		bfr = exp( (k1*cosp*cosp + k2*sinp*sinp)*sint*sint + k2*cost - k2 );
		if ((*theta) > HPAI) nsouth++;
/*		if ((*theta) > HPAI) fprintf(stderr, "%d\n", nsouth); */
	} while ((*theta)>HPAI || ran1(idum)>bfr);
	return nsouth;
}

/* ranh -- random sampling from homogeneous distribution on a sphere   2/24/98
	altered to "*idum = -1"   0699
	modified to (theta,phi) in radian   0819
*/

void ranh(theta, phi)
double *theta, *phi;
{
	static long idum[] = {-1};

	*theta = acos(1. - 2.*ran1(idum));
	*phi = DPAI*ran1(idum);
}
