#include <math.h>
#include "std.h"

/* rotvec -- rotates a vector alpha around z, beta around x   12/14/96
		A = (U)*A0 (a is unknown)
*/

void rotvec(alpha, beta, x0, y0, z0, x, y, z)
double alpha, beta, x0, y0, z0, *x, *y, *z;
{
	double cosa, sina, cosb, sinb;

	cosa = cos(alpha/RAD);
	sina = sin(alpha/RAD);
	cosb = cos(beta/RAD);
	sinb = sin(beta/RAD);
	*x = cosa*x0 - sina*y0;
	*y = sina*cosb*x0 + cosa*cosb*y0 - sinb*z0;
	*z = sina*sinb*x0 + cosa*sinb*y0 + cosb*z0;
}


/* rotuvec -- rotates a unit vector   12/16/96 */

void rotuvec(alpha, beta, inc0, dec0, inc, dec)
double alpha, beta, inc0, dec0, *inc, *dec;
{
	double cosa, sina, cosb, sinb, x0, y0, z0, x, y, z, dum;

	cosa = cos(alpha/RAD);   sina = sin(alpha/RAD);
	cosb = cos(beta/RAD);   sinb = sin(beta/RAD);
	magcar(1., inc0, dec0, &x0, &y0, &z0);
	x = cosa*x0 - sina*y0;
	y = sina*cosb*x0 + cosa*cosb*y0 - sinb*z0;
	z = sina*sinb*x0 + cosa*sinb*y0 + cosb*z0;
	carmag(x, y, z, &dum, inc, dec);
}

/* rotveceu -- rotate a vector around an Euler pole, adapted from fldadf   063021
	rotate a vector (plat0,plon0) around an Euler pole (elat,elon) by angle (delta)
	procedure:
		1. rotate a vector 90-elon around z toward y
		2. rotate the vector 90-elat around x toward z
		3. rotate the vector delta around z
		4. rotate the vector 90-elat around x away from z
		5. roteate the vector 90-elon around z away from y
*/

void rotveceu(elat, elon, plat0, plon0, delta, plat, plon)
double elat, elon, plat0, plon0, delta, *plat, *plon;
{
	double dum1, dum2, dum3, dum4;

	rotuvec(90.-elon, 90.-elat, plat0, plon0, &dum1, &dum2);
	rotuvec(delta, 0., dum1, dum2, &dum3, &dum4);
	rotuvec(0., elat-90., dum3, dum4, &dum1, &dum2);
	rotuvec(elon-90., 0., dum1, dum2, plat, plon);
}

/* rotveceu2 -- rotate a vector around an Euler pole (uses spherical triangulation)  070121
	Although mostly seems working well, not completely sure that the logic is valid.
	Hence, another one above "rotveceu" is recomended for use.
*/

#define EPS 1.e-4

void rotveceu2(elat, elon, plat0, plon0, delta, plat, plon)
double elat, elon, plat0, plon0, delta, *plat, *plon;
{
	double costhe, sinthe, costh0, sinth0, p, cosp, sinp, dlt0, cosdlt, sindlt, th, beta;

	if (elat<-90. || elat>90. || plat0<-90. || plat0>90.) error("Error of latitude!");
	if (elat == 90.){
		*plat = plat0;  *plon = plon0 + delta;
		checklon(plon, 0);
		return;
	}
	if (elat == -90.){
		*plat = plat0;  *plon = plon0 - delta;
		checklon(plon, 0);
		return;
	}
	costhe = cos((90.-elat)/RAD);  sinthe = sin((90.-elat)/RAD);
	checklon(&elon, 1);
	if (plat0 == 90.){
		cosp = costhe;  sinp = sinthe;
		cosdlt = cos(-delta/RAD);  sindlt = sin(-delta/RAD);
		th = acos(costhe*cosp + sinthe*sinp*cosdlt);
		*plat = 90. - th*RAD;
		if ((th<EPS && th>-EPS) || (th<PAI+EPS && th>PAI-EPS)) *plon = 0.;
		else {
			beta = asin(sinp*sindlt/sin(th));
			if (cosp >= costhe*cos(th)) *plon = elon + beta*RAD;
			else *plon = elon + 180. - beta*RAD;
		}
		checklon(plon, 0);
		return;
	}
	if (plat0 == -90.){
		cosp = -costhe;  sinp = sinthe;
		cosdlt = cos((180.-delta)/RAD);  sindlt = sin((180.-delta)/RAD);
		th = acos(costhe*cosp + sinthe*sinp*cosdlt);
		*plat = 90. - th*RAD;
		if ((th<EPS && th>-EPS) || (th<PAI+EPS && th>PAI-EPS)) *plon = 0.;
		else {
			beta = asin(sinp*sindlt/sin(th));
			if (cosp >= costhe*cos(th)) *plon = elon + beta*RAD;
			else *plon = elon + 180. - beta*RAD;
		}
		checklon(plon, 0);
		return;
	}
	costh0 = cos((90.-plat0)/RAD);  sinth0 = sin((90.-plat0)/RAD);
	checklon(&plon0, 1);
	p = acos(costh0*costhe + sinth0*sinthe*cos((plon0-elon)/RAD));
	cosp = cos(p);  sinp = sin(p);
	dlt0 = acos((costh0 - costhe*cosp)/(sinthe*sinp));
	if (plon0 < elon) dlt0 = -dlt0;
	cosdlt = cos(dlt0 - delta/RAD);  sindlt = sin(dlt0 - delta/RAD);
	th = acos(costhe*cosp + sinthe*sinp*cosdlt);
	*plat = 90. - th*RAD;
	if ((th<EPS && th>-EPS) || (th<PAI+EPS && th>PAI-EPS)) *plon = 0.;
	else {
		beta = asin(sinp*sindlt/sin(th));
		if (cosp >= costhe*cos(th)) *plon = elon + beta*RAD;
		else *plon = elon + 180. - beta*RAD;
	}
	checklon(plon, 0);
}
#undef EPS
