#include <math.h>
#include "std.h"

/* statf -- Fisher statistics of unit vectors   8/21/98,01/15/97,11/91,12/86
	Ver.5.0 -- the most simplified version without any flipping the reversed data
		   use fldnr() or vgpnr() as a pipe program to process data
		   when n=1, r, k, a95, and asd are set to -1
		   when nearly math error, a95 is set to either 0 or 180
	Ver.5.1 -- error of forgotten slat & slon were removed   070199
	Ver.6.0 -- ASD was removed and the older version was kept as statf2()   072599
	Ver.6.1 -- anglesd() and its families were moved to anglesd.c   122200
*/

void statf(inc, dec, n, incr, decr, r, k, a95)
int n;
double *inc, *dec, *incr, *decr, *r, *k, *a95;
{
	int i;
	double x, y, z, m, cosi, sini, cosd, sind, cosa, plat, sum, dum1, dum2;

	if (n <= 0) error("Error of n in statf");
	if (n == 1){
		*incr = inc[0];   *decr = dec[0];
		*r = -1.;   *k = -1.;   *a95 = -1.;
		return;
	}
	x = 0.;   y = 0.;   z = 0.;
	for (i = 0; i < n; i++){
		dum1 = inc[i]/RAD;   dum2 = dec[i]/RAD;
		cosi = cos(dum1);   sini = sin(dum1);
		cosd = cos(dum2);   sind = sin(dum2);
		x += cosi*cosd;   y += cosi*sind;   z += sini;
	}
	carmag(x, y, z, r, incr, decr);
	m = n;
	*k = (m - 1.)/(m - *r);
	cosa = 1.  -  (m - *r)/(*r)*(pow(20., 1./(m-1.)) - 1.);
	if (cosa > 1.) *a95 = 0.;
	else if (cosa < -1.) *a95 = 180.;
	else *a95 = acos(cosa)*RAD;
}

void statf2(inc, dec, n, incr, decr, r, k, a95, asd)
int n;
double *inc, *dec, *incr, *decr, *r, *k, *a95, *asd;
{
	int i;
	double x, y, z, m, cosi, sini, cosd, sind, cosa, plat, sum, dum1, dum2;

	if (n <= 0) error("Error of n in statf2");
	if (n == 1){
		*incr = inc[0];   *decr = dec[0];
		*r = -1.;   *k = -1.;   *a95 = -1.;   *asd = -1.;
		return;
	}
	x = 0.;   y = 0.;   z = 0.;
	for (i = 0; i < n; i++){
		dum1 = inc[i]/RAD;   dum2 = dec[i]/RAD;
		cosi = cos(dum1);   sini = sin(dum1);
		cosd = cos(dum2);   sind = sin(dum2);
		x += cosi*cosd;   y += cosi*sind;   z += sini;
	}
	carmag(x, y, z, r, incr, decr);
	m = n;
	*k = (m - 1.)/(m - *r);
	cosa = 1.  -  (m - *r)/(*r)*(pow(20., 1./(m-1.)) - 1.);
	if (cosa > 1.) *a95 = 0.;
	else if (cosa < -1.) *a95 = 180.;
	else *a95 = acos(cosa)*RAD;
	sum = 0.;
	cosi = cos(*incr/RAD);   sini = sin(*incr/RAD);
	for (i = 0; i < n; i++){
		dum1 = inc[i]/RAD;   dum2 = dec[i]/RAD;
		cosd = cos(dum1)*cosi*cos(dum2 - *decr/RAD) + sin(dum1)*sini;
		dum1 = acos(cosd);
		sum += dum1*dum1;
	}
	*asd = sqrt(sum/(m - 1.))*RAD;
}
