/* tmean -- test of a common mean of two scalar samples. Student's t-test is used
	when equal variances is supposed, otherwise, Welch's t-test is used.   092718
*/

#include <stdlib.h>
#include <math.h>
#include "tmean.h"

#define DL	1000.
#define HD	"Test of population means from two scalar samples:\n\
  Data file: %s,  Significance level: %6.4f,  Test if: mu1-mu2=%g?\n"
#define HDd	"Test of population means from two scalar samples:\n\
  Data file: %s,  Significance level: %6.4f,  Test if: mu1-mu2=%8.2e?\n"
#define HDF	"Null hypothesis H0; var1=var2 (Equal-Tails Test):\n"
#define CTF	"  F[%d,%d,%5.4f]  %s    p\n"
#define CTF1	"     %5.4f       %5.4f   %5.3f\n"
#define HDF1n	"  H0 was NOT REJECTED, proceed to Student's t-test supposing common variance.\n"
#define HDT	"Null hypothesis H0; mu1-mu2=%g when var1=var2 (Equal-Tails Test):\n"
#define HDTd	"Null hypothesis H0;mu1-mu2=%8.2e when var1=var2 (Equal-Tails Test):\n"
#define HDF1r	"  H0 was REJECTED, proceed to Welch's t-test when different variances.\n"
#define HDW	"Null hypothesis H0; mu1-mu2=%g when unequal variances (Equal-Tails Test):\n"
#define HDWd	"Null hypothesis H0; mu1-mu2=%8.2e when unequal variances (Equal-Tails Test):\n"
#define HDRn	"  H0 was NOT REJECTED.\n"
#define HDRr	"  H0 was REJECTED (NOT mu1-mu2=%g).\n"
#define HDRdr	"  H0 was REJECTED (NOT mu1-mu2=%8.2e).\n"
#define CTRT	"  tc[%.0f,%5.4f]      t        p\n"
#define CTRW	"  tc[%.2f,%5.4f]    t        p\n"
#define CTR1	"       %5.4f      %5.4f   %5.4f\n"
#define HDL	"Sample means:\n  Sample  N   Mean        Stdev       Variance\n"
#define CTL1	"     1  %3d  %#- 11.5g %#- 11.5g %#- 11.5g\n"
#define CTL2	"     2  %3d  %#- 11.5g %#- 11.5g %#- 11.5g\n"
#define HDL1	"Data of sample 1 & 2:\n    i     X1          X2\n"
#define CTL3	"  %3d  %#- 11.5g %#- 11.5g\n"
#define CTL4	"  %3d     -        %#- 11.5g\n"
#define CTL5	"  %3d  %#- 11.5g    -\n"

char Filed[NFILENAME];
unsigned int Opt = 0;
int N, N1, N2;
double Pc = 0.05, D = 0., *X1, *X2, M1, M2, V1, V2, S1, S2;

int main(argc, argv)
int argc;
char *argv[];
{
	FILE *fp;
	char strf[20];
	int df1, df2, i, il;
	double F, Fpc, p, dfd, t, tpc, pt;

	if ((fp = tmcmd(argc, argv)) == NULL) error("Error of file open!");
	tmread(fp);
	if (D<DL && D>-DL) fprintf(stdout, HD, Filed, Pc, D);
	else fprintf(stdout, HDd, Filed, Pc, D);
	meanvar(N1, X1, &M1, &V1, &S1);   meanvar(N2, X2, &M2, &V2, &S2);

	/* equal-tails test of common variance, then test of common mean */

	p = tmtestf(Pc/2., &df1, &df2, &Fpc, &F, strf);   /* F-test for common vars */
	fprintf(stdout, HDF);
	fprintf(stdout, CTF, df1, df2, Pc/2., strf);
	fprintf(stdout, CTF1, Fpc, F, p);
	if (p >= Pc/2.){                                  /* NOT rejected common vars */
		fprintf(stdout, HDF1n);
		if (D<DL && D>-DL) fprintf(stdout, HDT, D);
		else fprintf(stdout, HDTd, D);
		pt = tmtestt(Pc/2., &dfd, &tpc, &t);      /* Student's t-test */
		if (pt >= Pc/2.) fprintf(stdout, HDRn);
		else {
			if (D<DL && D>-DL) fprintf(stdout, HDRr, D);
			else fprintf(stdout, HDRdr, D);
		}
		fprintf(stdout, CTRT, dfd, Pc/2.);
		fprintf(stdout, CTR1, tpc, t, pt);
	}
	else {                                            /* REJECTED common vars */
		fprintf(stdout, HDF1r);
		if (D<DL && D>-DL) fprintf(stdout, HDW, D);
		else fprintf(stdout, HDWd, D);
		pt = tmtestw(Pc/2., &dfd, &tpc, &t);      /* Welch's t-test */
		if (pt >= Pc/2.) fprintf(stdout, HDRn);
		else {
			if (D<DL && D>-DL) fprintf(stdout, HDRr, D);
			else fprintf(stdout, HDRdr, D);
		}
		fprintf(stdout, CTRW, dfd, Pc/2.);
		fprintf(stdout, CTR1, tpc, t, pt);
	}

	/* print out means and standard deviations, and others. */

	fprintf(stdout, HDL);
	fprintf(stdout, CTL1, N1, M1, S1, V1);
	fprintf(stdout, CTL2, N2, M2, S2, V2);
	if (Opt & LIST_DATA){
		fprintf(stdout, HDL1);
		il = (N1 <= N2) ? N1 : N2;
		for (i=0; i<il; i++)
			fprintf(stdout, CTL3, i+1, X1[i], X2[i]);
		if (N1 < N2)
			for (i=il; i<N2; i++)
				fprintf(stdout, CTL4, i+1, X2[i]);
		else if (N1 > N2)
			for (i=il; i<N1; i++)
				fprintf(stdout, CTL5, i+1, X1[i]);
	}
	freevd(X1);   freevd(X2);
}
/*
for (i=0; i<N1; i++) printf("X1[%d]=%f\n", i, X1[i]);
for (i=0; i<N2; i++) printf("X2[%d]=%f\n", i, X2[i]);
printf("N1,M1,V1,N2,M2,V2: %d %6.3f %6.3f %d %6.3f %6.3f\n",N1,M1,V1,N2,M2,V2);

pt = tmtestw(Pc/2., &dfd, &tpc, &t); 
fprintf(stdout, "Welch tc[%.2f,%5.4f]    t        p\n", dfd, Pc/2.);
fprintf(stdout, "Welch      %5.4f      %5.4f   %5.4f\n", tpc, t, pt);
*/
