#include <math.h>
#include <string.h>
#include "tmean.h"

extern unsigned int Opt;
extern int N1, N2;
extern double Pc, D, M1, M2, V1, V2;

/* tmtestf -- equal-tails test of common variance using F distribution   092718
	Returns probability for the observed f to occur.
	NOTE: pc should be half of the specified significance level due to
	equal-tails test, even if f is calculated so that it is larger than unity.
*/

double tmtestf(pc, n1, n2, fc, f, s)
char *s;
int *n1, *n2;
double pc, *fc, *f;
{
	if (V1 > V2){
		*f = V1/V2;   strcpy(s, "var1/var2");
		*n1 = N1 - 1;  *n2 = N2 - 1;
	}
	else {
		*f = V2/V1;   strcpy(s, "var2/var1");
		*n1 = N2 - 1;  *n2 = N1 - 1;
	}
	*fc = ftab(*n1, *n2, pc);
	return fdist(*n1, *n2, *f);
}

/* tmtestt -- Student's t test (equal-tails test): returns p for the observed t to occur.
	Similarly with F-test, pc should be half of the specified significance level.
*/

double tmtestt(pc, nd, tc, t)
double pc, *nd, *tc, *t;
{
	double nd1, nd2, svar, dumt = 0.;

	nd1 = N1;   nd2 = N2;
	*nd = N1 + N2 - 2;
	*tc = ttab(*nd, pc);
	svar = ((nd1 - 1.)*V1 + (nd2 - 1.)*V2)/(*nd);
	*t = (M1 - M2 - D)/sqrt(svar*(1./nd1 + 1./nd2));
	return tdist(*nd, *t);
}

/* tmtestw -- Welch's t test (unequal variances t-test) (equal-tails test) */

double tmtestw(pc, nd, tc, t)
double pc, *nd, *tc, *t;
{
	double nd1, nd2, dum1, dum2, dum, dumt = 0.;

	nd1 = N1;   nd2 = N2;
	dum1 = V1/nd1;   dum2 = V2/nd2;   dum = dum1 + dum2;
	*nd = dum*dum/(dum1*dum1/(nd1-1.) + dum2*dum2/(nd2-1.));
	*tc = ttab(*nd, pc);
	*t = (M1 - M2 - D)/sqrt(dum);
	return tdist(*nd, *t);
}
