/* tmdread -- read data (directions) from a input file
	Variables:
		N -- total number of data (directions)
		M -- number of groups (distributions)
	Counters:
		i -- counts total number of data (directions)
		j -- counts total number of groups (distributions)
		k -- counts number of data for a group
		Nm[j] -- number of data of (j+1)th group.
			 there is no (M+1)th group, but Nm[M]=0 was assigned.
		Np[j] -- pointer to the 1st datum of (j+1)th group.
			 Np[M] points to the empty datum next to the last.
*/

#include <stdlib.h>
#include "tmeandir.h"

extern unsigned int Opt;
extern int N, M, *Nm, *Np;
extern double *Inc, *Dec, *Incr, *Decr, *R, *K, *A95;

void tmdread(fp)
FILE *fp;
{
	char s1[50], s2[50];
	int n, i = 0, j = 0, k = 0;

	Np[j] = i;
	while ((n = get_s(fp, 2, s1, s2)) != EOF){
		if (n==0 || s1[0]=='#' || s1[0]=='%')
			continue;
		if (s1[0]=='>' || s1[0]=='<'){
			if (k == 0) error("No data line before >");
			Nm[j] = k;
			Np[j+1] = Np[j] + Nm[j];
			j++;  k = 0;
			continue;
		}
		else {
			Inc[i] = atof(s1);   Dec[i] = atof(s2);
			i++;  k++;
		}
	}
	Nm[j] = k;   Nm[j+1] = 0;
	Np[j+1] = Np[j] + Nm[j];
	if (i != N) error("Error of data count!");
	if (++j != M) error("Error of group count!");
	if (M < 2) error("Only one group!");
	if (M == 2){
		if (Nm[0] < 3) error("Put at least 3 directions in 1st group!");
		if (Nm[0] < 5) Opt |= LESS_THAN_5;
		if (Nm[1]<1 || Nm[1]==2) error("Put at least 3 directions in 2nd group! (except 1 datum case)");
		if (Nm[1]==3 || Nm[1]==4) Opt |= LESS_THAN_5;
		if (Opt & INVERT_GRP2){
			for (i=Np[1]; i<Np[2]; i++){
				Inc[i] = -Inc[i];  Dec[i] = Dec[i] + 180.;
				checklon(Dec+i, 0);
			}
		}
	}
	else {
		if (Opt & INVERT_GRP2) error("Invalid R option for multi group case!");
		for (j=0; j<M; j++){
			if (Nm[j] < 3) error("Put at least 3 dirctions in each of multi group!");
			if (Nm[j] < 5) Opt |= LESS_THAN_5;
		}
	}
	fclose(fp);
}
