/* tmdstat1 -- test a single direction (group 2) is another observation from group 1 following
		McFadden & McElhinny (1990) and McFadden (1982).    090918
	NOTE: the formula was derived from (10) of M82, which is equivalent to (14) of M82 or
		(11) of MM90. These are NOT (19) of M82 which are for an outlier.
*/

#include <math.h>
#include <string.h>
#include "tmeandir.h"

extern int N, M, *Nm, *Np;
extern double Pc, *Inc, *Dec, *Incr, *Decr, *R, *K, *A95, Inct, Dect, Rt, Kt, A95t;

void tmdstat1()
{
	char sclass[20];
	int df1, df2;
	double cosgamm0, gamm0, gammc, gammcout, nd, G, Fpc, p, dum1, dum2;

	if (M!=2 || Nm[1]!=1) error("tmdstat1: error of M or Nm[1]!");
	statf(Inc, Dec, Nm[0], Incr, Decr, R, K, A95);
	statf(Inc, Dec, N, &Inct, &Dect, &Rt, &Kt, &A95t);
	Incr[1] = Inc[Np[1]];   Decr[1] = Dec[Np[1]];
	R[1] = 1.;   K[1] = -1.;   A95[1] = -1.;
	cosgamm0 = (Rt*Rt - R[0]*R[0] - 1.)/(2.*R[0]);
	gamm0 = RAD*acos(cosgamm0);
	nd = Nm[0];
	G = R[0]*(nd - 1.)*(1. - cosgamm0)/((R[0] + 1.)*(nd - R[0]));
	df1 = 2;  df2 = 2*(Nm[0] - 1);
	p = fdist(df1, df2, G);
	Fpc = ftab(df1, df2, Pc);
	gammc = RAD*acos(1. - ((R[0] + 1.)*(nd - R[0])/((nd - 1.)*R[0]))*Fpc);
	fprintf(stdout, "Null hypothesis H0; single group2 direction is from common mean group1:\n");
	fprintf(stdout, "  F[%d,%d,%5.3f]    G       p     gammc gamm0\n", df1, df2, Pc);
	fprintf(stdout, "     %5.4f      %5.4f  %5.3f  %5.1f %5.1f\n", Fpc, G, p, gammc, gamm0);
	tmdsetclass(gammc, sclass);
	if (p < Pc)
		fprintf(stdout, "  H0 was REJECTED (NEGATIVE reversal test).\n");
	else {
		fprintf(stdout, "  H0 was NOT REJECTED (POSITIVE reversal test).\n");
		fprintf(stdout, "    CLASS: %s\n", strcat(sclass, "i"));
	}
	dum1 = (R[0] + 1.)*(nd - R[0])/R[0];
	dum2 = 1. - pow(1. - Pc, 1./(nd + 1.));
	gammcout = RAD*acos(1. - dum1*(pow(1./dum2, 1./(nd - 1.)) - 1.));
	fprintf(stdout, "Test of outlier for the group2 single direction:\n");
	fprintf(stdout, "  gammc-out gamm0\n    %5.1f   %5.1f\n", gammcout, gamm0);
	if (gamm0 > gammcout) fprintf(stdout, "  OUTLIER, but not reject unless physical reasons for error are found.\n");
	else fprintf(stdout, "  NOT OUTLIER.\n");
}
/* (14) of McFadden (1982) naturally gives the same answer.
gammc = RAD*acos(1. - ((R[0] + 1.)*(nd - R[0])/R[0])*(pow(1./Pc, 1./(nd-1.)) - 1.));
gamm0 = angdif(1., Incr[1], Decr[1], 1., Incr[0], Decr[0]);
*/
