/* tmdstat2 -- statistics of McFadden_Lowes_81 for means of two distributions  091518,070799 */

#include <math.h>
#include <string.h>
#include "tmeandir.h"

extern unsigned int Opt;
extern int N, M, *Nm, *Np;
extern double Pc, *Inc, *Dec, *Incr, *Decr, *R, *K, *A95, Inct, Dect, Rt, Kt, A95t;

static double nd;

void tmdstat2()
{
	char sh[10], sclass[20];
	int j, df1, df2, dfr1, dfr2;
	double cosgamm0, gamm0, gammc, h, Fpc, pch, p, f;

	if (M != 2) error("tmdstat2: error of M!");
	for (j=0; j<M; j++)
		statf(Inc+Np[j], Dec+Np[j], Nm[j], Incr+j, Decr+j, R+j, K+j, A95+j);
	statf(Inc, Dec, N, &Inct, &Dect, &Rt, &Kt, &A95t);
	cosgamm0 = (Rt*Rt - R[0]*R[0] - R[1]*R[1])/(2.*R[0]*R[1]);
	gamm0 = RAD*acos(cosgamm0);

/* equal-tails test of common kappa using h (1/h) of (24a), and one-sided test of common mean using (25) */

	pch = Pc/2.;
	if (K[1] > K[0]){
		h = K[1]/K[0];   strcpy(sh, "k2/k1");
		dfr1 = 2*(Nm[0]-1);   dfr2 = 2*(Nm[1]-1);
	}
	else {
		h = K[0]/K[1];   strcpy(sh, "k1/k2");
		dfr1 = 2*(Nm[1]-1);   dfr2 = 2*(Nm[0]-1);
	}
	Fpc = ftab(dfr1, dfr2, pch);
	p = fdist(dfr1, dfr2, h);
	fprintf(stdout, "Null hypothesis H0; kappa1 = kappa2 (Equal-Tails Test):\n");
	fprintf(stdout, "  F[%d,%d,%5.4f]  %s    p\n", dfr1, dfr2, pch, sh);
	fprintf(stdout, "     %5.4f       %5.4f  %5.3f\n", Fpc, h, p);
	if (p>=pch && (Opt & LESS_THAN_5)){
		fprintf(stdout, "  H0 was NOT REJECTED, but a group with less than 5 data exists!\n  Simulation follows supposing common kappa.\n");
		tmdstat2s(cosgamm0);
	}
	else if (p >= pch){
		fprintf(stdout, "  H0 was NOT REJECTED, proceed with test supposing common kappa.\n");
		fprintf(stdout, "Null hypothesis H0; common true mean when kappa1 = kappa2:\n");
		f = tmdcalf(N, R[0], R[1], Rt);
		df1 = 2;   df2 = 2*(N - 2);
		p = fdist(df1, df2, f);
		Fpc = ftab(df1, df2, Pc);
		gammc = RAD*acos(tmdcosgammc(N, R[0], R[1], Fpc));
		fprintf(stdout, "  F[%d,%d,%5.3f]    f      p      gammc gamm0\n", df1, df2, Pc);
		fprintf(stdout, "     %5.4f      %5.4f  %5.3f  %5.1f %5.1f\n", Fpc, f, p, gammc, gamm0);
		tmdsetclass(gammc, sclass);
		if (p < Pc)
			fprintf(stdout, "  H0 was REJECTED (NEGATIVE reversal test).\n");
		else {
			fprintf(stdout, "  H0 was NOT REJECTED (POSITIVE reversal test).\n");
			fprintf(stdout, "    CLASS: %s\n", sclass);
		}
/*
		fprintf(stdout, "  Simulation will also be presented for kappa1 = kappa2.\n");
		tmdstat2s(cosgamm0);
*/
	}
	else {
		fprintf(stdout, "  H0 was REJECTED. Simulation follows when kappa1 != kappa2.\n");
		tmdstat2s(cosgamm0);
	}
}

/* tmdcalf -- left-hand side of (25) */

double tmdcalf(n, R1, R2, R)
int n;
double R1, R2, R;
{
	nd = n;
	return 0.5*(nd - 2.)*(R1 + R2 - R*R/(R1 + R2))/(nd - R1 - R2);
}

/* tmdcosgammc -- returns cosine of gammac between the two means using (25) (NOT (29)) */

double tmdcosgammc(n, R1, R2, Fpc)
int n;
double R1, R2, Fpc;
{
	nd = n;
	return 1. - (nd - R1 -R2)*(R1 + R2)*Fpc/((nd - 2.)*R1*R2);
}

/* tmdsetclass -- classification of reversal test by Mcfadden & McElhinny 1990 */

void tmdsetclass(ang, s)
char *s;
double ang;
{
	if (ang <= 5.) strcpy(s, "A");
	else if (ang <= 10.) strcpy(s, "B");
	else if (ang <= 20.) strcpy(s, "C");
	else strcpy(s, "INDETERMINATE");
}
