/* tmdstat2s -- simulation common mean test for two groups following MM90   091118 */

#include <math.h>
#include "tmeandir.h"

extern int NI, *Nm, *Np;
extern double Pc, *Inc, *Dec, *R, *K;

void tmdstat2s(cosgamm0)
double cosgamm0;
{
	char sclass[20];
	int i, ic;
	double Sr0, Rw0, V0, V[NI], cosgamm[NI], Vc, gammc, dum1, dum2;

	dum1 = K[0]*R[0];   dum2 = K[1]*R[1];
	Sr0 = dum1 + dum2;
	Rw0 = sqrt(dum1*dum1 + dum2*dum2 + 2.*dum1*dum2*cosgamm0);
	V0 = 2.*(Sr0 - Rw0);
	for (i=0; i<NI; i++){
		V[i] = calranv(Nm[0], Nm[1], K[0], K[1], cosgamm+i);
	}
	sort2(NI, V, cosgamm);
	ic = NI*(1. - Pc) - 1;
	Vc = V[ic];   gammc = RAD*acos(cosgamm[ic]);
	tmdsetclass(gammc, sclass);
	fprintf(stdout,"Test by simulation (ni=%d) -- Null hypothesis H0; common true mean:\n",NI);
	fprintf(stdout, "  Vc(p=%.3f)   V0     gammc gamm0\n    %5.4f    %5.4f  %5.1f %5.1f\n", \
		Pc, Vc, V0, RAD*acos(cosgamm[ic]), RAD*acos(cosgamm0));
	if (V0 > Vc)
		fprintf(stdout, "  H0 was REJECTED (NEGATIVE reversal test).\n");
	else {
		fprintf(stdout, "  H0 was NOT REJECTED (POSITIVE reversal test).\n");
		fprintf(stdout, "    CLASS: %s\n", sclass);
	}
}

/* calranv -- return V of two random Fisher distributions with specified n1, n2, k1, k2.
	NOTE:
	It might be correct that k1 & k2 are also simulated values as k1=(n1-1)/(n1-r1).
	However, in that way correlation of i-th V & i-th gamma becomes very bad and
	erratic. Hence, observation values were used for k1 & k2. See README in zabondoned/.
*/

double calranv(n1, n2, k1, k2, cosgm)
int n1, n2;
double k1, k2, *cosgm;
{
	int i;
	double th, ph, sint, x1, y1, z1, x2, y2, z2, r1, r2, sr, xh, yh, zh, rw, dum1, dum2;

	x1 = y1 = z1 = x2 = y2 = z2 = 0.;
	for (i=0; i<n1; i++){
		ranf2(k1, &th, &ph);
		sint = sin(th);
		x1 += sint*cos(ph);   y1 += sint*sin(ph);   z1 += cos(th);
	}
	r1 = sqrt(x1*x1 + y1*y1 + z1*z1);
	for (i=0; i<n2; i++){
		ranf2(k2, &th, &ph);
		sint = sin(th);
		x2 += sint*cos(ph);   y2 += sint*sin(ph);   z2 += cos(th);
	}
	r2 = sqrt(x2*x2 + y2*y2 + z2*z2);
	sr = k1*r1 + k2*r2;
	xh = k1*x1 + k2*x2;   yh = k1*y1 + k2*y2;   zh = k1*z1 + k2*z2;
	rw = sqrt(xh*xh + yh*yh + zh*zh);
	dum1 = k1*r1;   dum2 = k2*r2;
	*cosgm = (rw*rw - dum1*dum1 - dum2*dum2)/(2.*dum1*dum2);
	return 2.*(sr - rw);
}
/*
sort(NI, V);   sort(NI, cosgamm);
for (i=0; i<NI; i++) printf("i,V,gamm: %3d %7.3f %6.2f\n", i, V[i], RAD*acos(cosgamm[i]));

fprintf(stdout, "  Vc(p=%.3f)   V0     gammc gamm0\n    %5.4f    %5.4f  %5.1f %5.1f\n", \
Pc, Vc, V0, 180.-RAD*acos(cosgamm[ic]), RAD*acos(cosgamm0));

*cosgm = -(rw*rw - dum1*dum1 - dum2*dum2)/(2.*dum1*dum2);
*/
