/* tmdstatm -- test of common true mean for multi groups after McFadden & Lowes (1981)   070799
		rewritten for use in tmeandir   091418
*/

#include <math.h>
#include "tmeandir.h"

extern unsigned int Opt;
extern int N, M, *Nm, *Np;
extern double Pc, *Inc, *Dec, *Incr, *Decr, *R, *K, *A95, Inct, Dect, Rt, Kt, A95t;

static int j;
static double nd, md;

void tmdstatm()
{
	int df1, df2;
	double g, Fc, p, Km, G, pG, Chi2c;

	if (M <= 2) error("tmdstatm: error of M!");
	for (j=0; j<M; j++)
		statf(Inc+Np[j], Dec+Np[j], Nm[j], Incr+j, Decr+j, R+j, K+j, A95+j);
	statf(Inc, Dec, N, &Inct, &Dect, &Rt, &Kt, &A95t);

	/* test common kappa using (64), (65), (66), if not rejected, test common mean by (41) */

	Km = tmdcalk(N, M, Nm, R);
	G = tmdcalG(N, M, Nm, K, Km);
	Chi2c = chi2tab(M-1, Pc);
	pG = chi2dist(M-1, G);
	fprintf(stdout, "Null hypothesis H0: common kappa:\n");
	fprintf(stdout, "  Chi2[%d,%5.3f]   G      p\n", M-1, Pc);
	fprintf(stdout, "    %7.4f    %7.4f  %5.3f\n", Chi2c, G, pG);
	if (pG>=Pc && (Opt & LESS_THAN_5)){
		fprintf(stdout, "  H0 was NOT REJECTED, but a group with less than 5 data exists!\n  Simulation follows supposing common kappa.\n");
		tmdstatms();
	}
	else if (pG >= Pc){
		fprintf(stdout, "  H0 was NOT REJECTED, proceed with test supposing common kappa.\n");
		g = tmdcalg(N, M, R, Rt);
		df1 = 2*(M-1);   df2 = 2*(N-M);
		Fc = ftab(df1, df2, Pc);
		p = fdist(df1, df2, g);
		fprintf(stdout, "Null hypothesis H0: common true mean when common kappa:\n");
		fprintf(stdout, "  F[%d,%d,%5.3f]   g      p\n", df1, df2, Pc);
		fprintf(stdout, "     %5.4f     %5.4f  %5.3f\n", Fc, g, p);
		if (p < Pc)
			fprintf(stdout, "  H0 was REJECTED (NOT common mean).\n");
		else
			fprintf(stdout, "  H0 was NOT REJECTED.\n");
/*
		fprintf(stdout, "  Simulation will also be presented for common kappa.\n");
		tmdstatms();
*/
	}
	else {
		fprintf(stdout, "  H0 was REJECTED. Simulation follows when NOT common kappa.\n");
		tmdstatms();
	}
}

/* tmdcalk -- inverse of pooled unviased estimate of (1/kappa) given by (64) */

double tmdcalk(n, m, nm, r)
int n, m, *nm;
double *r;
{
	double sum = 0.;

	nd = n;   md = m;
	for (j=0; j<m; j++) sum += (double)nm[j] - r[j];
	return (nd - md)/sum;
}

/* tmdcalG -- calculate G of (65) */

double tmdcalG(n, m, nm, k, km)
int n, m, *nm;
double *k, km;
{
	double sum1 = 0., sum2 = 0., c;

	nd = n;   md = m;
	for (j=0; j<m; j++){
		sum1 += ((double)nm[j] - 1.)*log(k[j]);
		sum2 += 1./((double)nm[j] - 1.);
	}
	c = 1. + (sum2 - 1./(nd - md))/(6.*(md - 1.));
	return 2.*(sum1 - (nd - md)*log(km))/c;
}

/* tmdcalg -- left-hand side, g, of (41) */

double tmdcalg(n, m, r, rt)
int n, m;
double *r, rt;
{
	double sumr = 0.;

	nd = n;   md = m;
	for (j=0; j<m; j++) sumr += r[j];
	return (nd - md)*(sumr - rt*rt/sumr)/((md - 1.)*2.*(nd - sumr));
}
