/* tmdstatms -- simulation common mean test for multi groups with not all equal kappas.  This is
	based on the formulas at p.211 of book Fisher et al. (1987). The original is only
	applicable for large number of data (n>25), but simulation with random numbers should
	work as the method of McFadden & McElhinny (1990) for two groups case.   091518
*/

#include <math.h>
#include "tmeandir.h"

extern int M, *Nm, NI;
extern double Pc, *Inc, *Dec, *Incr, *Decr, *R, *K;

void tmdstatms()
{
	int i, j, ic;
	double Sr0, Rw0, V0, Xh0, Yh0, Zh0, V[NI], Vc, dum1, dum2, dum3;

	Sr0 = Xh0 = Yh0 = Zh0 = 0.;
	for (j=0; j<M; j++){
		Sr0 += K[j]*R[j];
		magcar(R[j], Incr[j], Decr[j], &dum1, &dum2, &dum3);
		Xh0 += K[j]*dum1;   Yh0 += K[j]*dum2;   Zh0 += K[j]*dum3;
	}
	Rw0 = sqrt(Xh0*Xh0 + Yh0*Yh0 + Zh0*Zh0);
	V0 = 2.*(Sr0 - Rw0);
	for (i=0; i<NI; i++){
		V[i] = calranvm();
	}
	sort(NI, V);
	ic = NI*(1. - Pc) - 1;
	Vc = V[ic];
	fprintf(stdout,"Test by simulation (ni=%d) -- Null hypothesis H0; common true mean:\n",NI);
	fprintf(stdout, "  Vc(p=%.3f)   V0\n    %5.4f    %5.4f\n", Pc, Vc, V0);
	if (V0 > Vc)
		fprintf(stdout, "  H0 was REJECTED (NOT common mean).\n");
	else
		fprintf(stdout, "  H0 was NOT REJECTED.\n");
}

/* calranvm -- return V of M random Fisher distributions from Nm[], K[], Inc[], Dec[], ...
	NOTE:
	As the best estimate of random kappa, an observed value K[] was used,
	not a simulated k. For the reasons, see the note in tmdstat2s and README in zabondoned\.
*/

double calranvm()
{
	int j, k;
	double sr, rw, x, y, z, th, ph, sint, xk, yk, zk, rk;

	sr = x = y = z = 0.;
	for (j=0; j<M; j++){
		xk = yk = zk = 0.;
		for (k=0; k<Nm[j]; k++){
			ranf2(K[j], &th, &ph);
			sint = sin(th);
			xk += sint*cos(ph);   yk += sint*sin(ph);   zk += cos(th);
		}
		rk = sqrt(xk*xk + yk*yk + zk*zk);
		sr += K[j]*rk;
		x += K[j]*xk;   y += K[j]*yk;   z += K[j]*zk;
	}
	rw = sqrt(x*x + y*y + z*z);
	return 2.*(sr - rw);
}
/*
printf("Xh0,Yh0,Zh0,Sr0,Rw0,V0: %6.4f %6.4f %6.4f %6.4f %6.4f %6.4f\n",Xh0,Yh0,Zh0,Sr0,Rw0,V0);
printf("i,V: %3d %7.3f\n", i, V[i]);
for (i=0; i<NI; i++) printf("i,V: %3d %7.3f\n", i, V[i]);
printf("k,K,th,ph: %d %8.4f %5.3f %5.3f\n", k, K[j], th, ph);
printf("j,xk,yk,zk,rk: %d %6.4f %6.4f %6.4f %6.4f\n", j, xk, yk, zk, rk);
printf("x,y,z,sr,rw: %6.4f %6.4f %6.4f %6.4f %6.4f\n", x, y, z, sr, rw);
*/
